@@ -4,7 +4,7 @@ package gnu.dtools.ritopt;
  * LongOption.java
  *
  * Version:
- *    $Id: LongOption.java 322 2004-05-20 20:45:47Z mortenalver $
+ *    $Id: LongOption.java 2488 2007-11-14 00:25:31Z coezbek $
  */
 
 /**
@@ -46,7 +46,7 @@ public class LongOption extends Option {
      */
 
     public LongOption() {
-	this( (long)0 );
+	this( 0 );
     }
 
     /**
