@@ -4,10 +4,12 @@ package gnu.dtools.ritopt;
  * OptionMenu.java
  *
  * Version:
- *    $Id: OptionMenu.java 322 2004-05-20 20:45:47Z mortenalver $
+ *    $Id: OptionMenu.java 2209 2007-08-01 18:23:38Z coezbek $
  */
 
-import java.io.*;
+import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStreamReader;
 
 /**
  * This class facilitates the built-in menu feature for ritopt. When the
