@@ -123,7 +123,9 @@ public class AuthorList {
 
 	private String[] authorsFirstFirst = new String[4], authorsLastOnly = new String[2],
 	authorLastFirstAnds = new String[2], 
-	authorsLastFirst = new String[4];
+	authorsLastFirst = new String[4],
+    authorsLastFirstFirstLast = new String[2];
+
 
 	// The following variables are used only during parsing
 
@@ -243,7 +245,7 @@ public class AuthorList {
 	 * @return An AuthorList object representing the given authors.
 	 */
 	public static AuthorList getAuthorList(String authors) {
-		AuthorList authorList = (AuthorList) authorCache.get(authors);
+		AuthorList authorList = authorCache.get(authors);
 		if (authorList == null) {
 			authorList = new AuthorList(authors);
 			authorCache.put(authors, authorList);
@@ -385,9 +387,16 @@ public class AuthorList {
 			if (von_start < 0) { // no 'von part'
 				last_part_end = tokens.size();
 				last_part_start = tokens.size() - TOKEN_GROUP_LENGTH;
+				int index = tokens.size() - 2 * TOKEN_GROUP_LENGTH + OFFSET_TOKEN_TERM;
+				if (index > 0) {
+					Character ch = (Character)tokens.elementAt(index);
+					if (ch.charValue() == '-')
+						last_part_start -= TOKEN_GROUP_LENGTH;
+				}
 				first_part_end = last_part_start;
-				if (first_part_end > 0)
+				if (first_part_end > 0) {
 					first_part_start = 0;
+				}
 			} else { // 'von part' is present
 				if (last_start >= 0) {
 					last_part_end = tokens.size();
@@ -430,6 +439,17 @@ public class AuthorList {
 			}
 		}
 
+        if ((first_part_start == -1) && (last_part_start == -1) && (von_part_start != -1)) {
+            // There is no first or last name, but we have a von part. This is likely
+            // to indicate a single-entry name without an initial capital letter, such
+            // as "unknown".
+            // We make the von part the last name, to facilitate handling by last-name formatters:
+            last_part_start = von_part_start;
+            last_part_end = von_part_end;
+            von_part_start = -1;
+            von_part_end = -1;
+        }
+
 		// Third step: do actual splitting, construct Author object
 		return new Author((first_part_start < 0 ? null : concatTokens(first_part_start,
 			first_part_end, OFFSET_TOKEN, false)), (first_part_start < 0 ? null : concatTokens(
@@ -552,12 +572,13 @@ public class AuthorList {
 			if (c == '\\')
 				current_backslash = token_end;
 			if (braces_level == 0)
-				if (c == ',' || c == '~' || /* c=='-' || */Character.isWhitespace(c))
+				if (c == ',' || c == '~' || c=='-' || Character.isWhitespace(c))
 					break;
 			// Morten Alver 18 Apr 2006: Removed check for hyphen '-' above to
 			// prevent
 			// problems with names like Bailey-Jones getting broken up and
 			// sorted wrong.
+			// Aaron Chen 14 Sep 2008: Enable hyphen check for first names like Chang-Chin
 			token_end++;
 		}
 		if (token_abbr < 0)
@@ -587,7 +608,7 @@ public class AuthorList {
 	 * @return the <CODE>Author</CODE> object.
 	 */
 	public Author getAuthor(int i) {
-		return (Author) authors.get(i);
+		return authors.get(i);
 	}
 
 	/**
@@ -643,7 +664,6 @@ public class AuthorList {
 	 * @return formatted list of authors.
 	 */
 	public String getAuthorsLastOnly(boolean oxfordComma) {
-
 		int abbrInt = (oxfordComma ? 0 : 1);
 
 		// Check if we've computed this before:
@@ -744,7 +764,6 @@ public class AuthorList {
 	 * @return formatted list of authors.
 	 */
 	public String getAuthorsLastFirstAnds(boolean abbreviate) {
-
 		int abbrInt = (abbreviate ? 0 : 1);
 		// Check if we've computed this before:
 		if (authorLastFirstAnds[abbrInt] != null)
@@ -763,6 +782,25 @@ public class AuthorList {
 		return authorLastFirstAnds[abbrInt];
 	}
 
+	public String getAuthorsLastFirstFirstLastAnds(boolean abbreviate) {
+		int abbrInt = (abbreviate ? 0 : 1);
+		// Check if we've computed this before:
+		if (authorsLastFirstFirstLast[abbrInt] != null)
+			return authorsLastFirstFirstLast[abbrInt];
+
+		StringBuffer res = new StringBuffer();
+		if (size() > 0) {
+            res.append(getAuthor(0).getLastFirst(abbreviate));
+			for (int i = 1; i < size(); i++) {
+				res.append(" and ");
+				res.append(getAuthor(i).getFirstLast(abbreviate));
+			}
+		}
+
+		authorsLastFirstFirstLast[abbrInt] = res.toString();
+		return authorsLastFirstFirstLast[abbrInt];
+	}    
+
 	/**
 	 * Returns the list of authors separated by commas with first names before
 	 * last name; first names are abbreviated or not depending on parameter. If
@@ -1001,7 +1039,7 @@ public class AuthorList {
 		 * @return last name of the author (may consist of several tokens)
 		 */
 		public String getLast() {
-			return last_part;
+            return last_part;
 		}
 
 		/**
