@@ -29,10 +29,11 @@ Modified for use in JabRef.
 */
 package net.sf.jabref;
 
-import java.io.*;
-import java.util.*;
+import java.util.Iterator;
+import java.util.TreeMap;
+import java.util.Locale;
 
-public abstract class BibtexEntryType implements Comparable
+public abstract class BibtexEntryType implements Comparable<BibtexEntryType>
 {
 
     public static final BibtexEntryType OTHER =
@@ -59,7 +60,7 @@ public abstract class BibtexEntryType implements Comparable
                 return "";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return true;
             }
@@ -95,12 +96,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, JOURNAL and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "author", "title", "journal", "year", "bibtexkey", "volume", "pages"
-                    });
+                    }, database);
             }
         };
 
@@ -133,12 +134,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "TITLE";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "title", "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -175,12 +176,12 @@ public abstract class BibtexEntryType implements Comparable
 		    +"EDITOR and/or AUTHOR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "title", "publisher", "year", "bibtexkey"
-                    }) &&
+                    }, database) &&
 		    (((entry.getField("author") != null) ||
 		      (entry.getField("editor") != null)) &&
 		     ((entry.getField("chapter") != null) ||
@@ -218,12 +219,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "TITLE, PUBLISHER, YEAR, and an EDITOR and/or AUTHOR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "title", "publisher", "year", "bibtexkey"
-                    }) &&
+                    }, database) &&
                 ((entry.getField("author") != null) ||
                 (entry.getField("editor") != null));
             }
@@ -260,14 +261,14 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, BOOKTITLE, PUBLISHER and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
 			"author", "title", "booktitle", "publisher", "year",
 			"bibtexkey"
 
-                    });
+                    }, database);
             }
         };
 
@@ -301,12 +302,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, BOOKTITLE and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
 			"author", "title", "booktitle", "year" , "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -340,12 +341,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, BOOKTITLE and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
 			"author", "title", "booktitle", "year" , "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -379,12 +380,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "TITLE and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
 			"title", "year", "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -419,12 +420,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "TITLE";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "title", "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -457,13 +458,13 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, INSTITUTION and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
 			"author", "title", "institution", "year",
 			"bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -497,12 +498,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, SCHOOL and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "author", "title", "school", "year", "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -536,12 +537,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE, SCHOOL and YEAR";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
                         "author", "title", "school", "year", "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -574,15 +575,170 @@ public abstract class BibtexEntryType implements Comparable
                 return "AUTHOR, TITLE and NOTE";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
                 return entry.allFieldsPresent(new String[]
                     {
 			"author", "title", "note", "bibtexkey"
-                    });
+                    }, database);
             }
         };
 
+     public static final BibtexEntryType PERIODICAL =
+        new BibtexEntryType()
+        {
+            public String getName()
+            {
+                return "Periodical";
+            }
+
+            public String[] getOptionalFields()
+            {
+                return new String[]
+                {
+                    "editor", "language", "series", "volume", "number", "organization", "month", "note", "url"
+                };
+            }
+
+            public String[] getRequiredFields()
+            {
+                return new String[]
+                {
+                    "title", "year"
+                };
+            }
+
+            public String describeRequiredFields()
+            {
+                return "TITLE and YEAR";
+            }
+
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
+            {
+                return entry.allFieldsPresent(new String[]
+                    {
+                        "title", "year", "bibtexkey"
+                    }, database);
+            }
+        };
+
+     public static final BibtexEntryType PATENT =
+        new BibtexEntryType()
+        {
+            public String getName()
+            {
+                return "Patent";
+            }
+
+            public String[] getOptionalFields()
+            {
+                return new String[]
+                {
+                    "author", "title", "language", "assignee", "address", "type", "number", "day", "dayfiled", "month", "monthfiled", "note", "url"
+                };
+            }
+
+            public String[] getRequiredFields()
+            {
+                return new String[]
+                {
+                    "nationality", "number", "year", "yearfiled"
+                };
+            }
+
+            public String describeRequiredFields()
+            {
+                return "NATIONALITY, NUMBER, YEAR or YEARFILED";
+            }
+
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
+            {
+                return entry.allFieldsPresent(new String[]
+                    {
+                        "number", "bibtexkey"
+                    }, database) &&
+                ((entry.getField("year") != null) ||
+                (entry.getField("yearfiled") != null));
+            }
+        };
+
+   public static final BibtexEntryType STANDARD =
+        new BibtexEntryType()
+        {
+            public String getName()
+            {
+                return "Standard";
+            }
+
+            public String[] getOptionalFields()
+            {
+                return new String[]
+                {
+                    "author", "language", "howpublished", "type", "number", "revision", "address", "month", "year", "note", "url"
+                };
+            }
+
+            public String[] getRequiredFields()
+            {
+                return new String[]
+                {
+                    "title", "organization", "institution"
+                };
+            }
+
+            public String describeRequiredFields()
+            {
+                return "TITLE, ORGANIZATION or INSTITUTION";
+            }
+
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
+            {
+                return entry.allFieldsPresent(new String[]
+                    {
+                        "title", "bibtexkey"
+                    }, database) &&
+                ((entry.getField("organization") != null) ||
+                (entry.getField("institution") != null));
+            }
+        };
+
+    public static final BibtexEntryType ELECTRONIC =
+        new BibtexEntryType()
+        {
+            public String getName()
+            {
+                return "Electronic";
+            }
+
+            public String[] getOptionalFields()
+            {
+                return new String[]
+                {
+                    "author", "month", "year", "title", "language", "howpublished", "organization", "address", "note", "url"
+                };
+            }
+
+            public String[] getRequiredFields()
+            {
+                return null;
+            }
+
+            public String describeRequiredFields()
+            {
+                return "None";
+            }
+
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
+            {
+                return entry.allFieldsPresent(new String[]
+                    {
+                        "bibtexkey"
+                    }, database) &&
+                ((entry.getField("howpublished") != null) ||
+                (entry.getField("note") != null) ||
+                (entry.getField("url") != null));
+            }
+        };
 
     public static final BibtexEntryType MISC =
         new BibtexEntryType()
@@ -610,12 +766,12 @@ public abstract class BibtexEntryType implements Comparable
                 return "None";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
 		return entry.allFieldsPresent(new String[]
                     {
 			"bibtexkey"
-                    });
+                    }, database);
             }
         };
 
@@ -646,7 +802,7 @@ public abstract class BibtexEntryType implements Comparable
                 return "None";
             }
 
-            public boolean hasAllRequiredFields(BibtexEntry entry)
+            public boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database)
             {
 		return false;
            }
@@ -655,8 +811,8 @@ public abstract class BibtexEntryType implements Comparable
 
     public abstract String getName();
 
-    public int compareTo(Object o) {
-	return getName().compareTo(((BibtexEntryType)o).getName());
+    public int compareTo(BibtexEntryType o) {
+	return getName().compareTo(o.getName());
     }
 
     public abstract String[] getOptionalFields();
@@ -665,13 +821,14 @@ public abstract class BibtexEntryType implements Comparable
 
     public String[] getGeneralFields() {
         return new String[]
-	    {"crossref", "keywords", "doi", "url",
-	     "citeseerurl", "pdf", "abstract", "comment"};
+	    {"crossref", "keywords", "doi", "url", "file",
+	     "citeseerurl", "pdf", "abstract", "comment",
+         "owner", "timestamp", "review", };
     }
 
     public abstract String describeRequiredFields();
 
-    public abstract boolean hasAllRequiredFields(BibtexEntry entry);
+    public abstract boolean hasAllRequiredFields(BibtexEntry entry, BibtexDatabase database);
 
 
     public String[] getUtilityFields(){
@@ -695,8 +852,8 @@ public abstract class BibtexEntryType implements Comparable
 	return false;
     }
 
-    public static TreeMap ALL_TYPES = new TreeMap();
-    public static TreeMap STANDARD_TYPES = new TreeMap();
+    public static TreeMap<String, BibtexEntryType> ALL_TYPES = new TreeMap<String, BibtexEntryType>();
+    public static TreeMap<String, BibtexEntryType> STANDARD_TYPES = new TreeMap<String, BibtexEntryType>();
     static {
 	// Put the standard entry types into the type map.
 	ALL_TYPES.put("article", ARTICLE);
@@ -704,7 +861,7 @@ public abstract class BibtexEntryType implements Comparable
 	ALL_TYPES.put("book", BOOK);
 	ALL_TYPES.put("booklet", BOOKLET);
 	ALL_TYPES.put("incollection", INCOLLECTION);
-        ALL_TYPES.put("conference", CONFERENCE);
+    ALL_TYPES.put("conference", CONFERENCE);
 	ALL_TYPES.put("inproceedings", INPROCEEDINGS);
 	ALL_TYPES.put("proceedings", PROCEEDINGS);
 	ALL_TYPES.put("manual", MANUAL);
@@ -712,12 +869,16 @@ public abstract class BibtexEntryType implements Comparable
 	ALL_TYPES.put("phdthesis", PHDTHESIS);
 	ALL_TYPES.put("techreport", TECHREPORT);
 	ALL_TYPES.put("unpublished", UNPUBLISHED);
-        ALL_TYPES.put("misc", MISC);
-        ALL_TYPES.put("other", OTHER);
+	ALL_TYPES.put("patent", PATENT);
+	ALL_TYPES.put("standard", STANDARD);
+	ALL_TYPES.put("electronic", ELECTRONIC);
+	ALL_TYPES.put("periodical", PERIODICAL);
+    ALL_TYPES.put("misc", MISC);
+    ALL_TYPES.put("other", OTHER);
 
 	// We need a record of the standard types, in case the user wants
 	// to remove a customized version. Therefore we clone the map.
-	STANDARD_TYPES = (TreeMap)ALL_TYPES.clone();
+	STANDARD_TYPES = new TreeMap<String, BibtexEntryType>(ALL_TYPES);
     }
 
     /**
@@ -726,7 +887,7 @@ public abstract class BibtexEntryType implements Comparable
      */
     public static BibtexEntryType getType(String name) {
 	//Util.pr("'"+name+"'");
-	Object o = ALL_TYPES.get(name.toLowerCase());
+	Object o = ALL_TYPES.get(name.toLowerCase(Locale.US));
 	if (o == null)
 	    return null;
 	else return (BibtexEntryType)o;
@@ -783,7 +944,7 @@ public abstract class BibtexEntryType implements Comparable
      * JabRefFrame when the program closes.
      */
     public static void saveCustomEntryTypes(JabRefPreferences prefs) {
-	Iterator i=ALL_TYPES.keySet().iterator();
+	Iterator<String> i=ALL_TYPES.keySet().iterator();
 	int number = 0;
 	//Vector customTypes = new Vector(10, 10);
 	while (i.hasNext()) {
