@@ -26,34 +26,47 @@ package net.sf.jabref;
 
 import java.awt.*;
 import java.awt.datatransfer.StringSelection;
-import java.awt.dnd.*;
+import java.awt.dnd.DnDConstants;
+import java.awt.dnd.DropTarget;
 import java.awt.event.*;
-import java.beans.*;
-import java.io.*;
-import java.util.*;
+import java.beans.PropertyChangeEvent;
+import java.beans.VetoableChangeListener;
+import java.io.File;
+import java.io.IOException;
+import java.io.StringWriter;
+import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.logging.Logger;
 
 import javax.swing.*;
-import javax.swing.event.*;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
 import javax.swing.text.JTextComponent;
 
 import net.sf.jabref.export.LatexFieldFormatter;
-import net.sf.jabref.groups.*;
-import net.sf.jabref.imports.BibtexParser;
-import net.sf.jabref.labelPattern.LabelPatternUtil;
-import net.sf.jabref.undo.*;
 import net.sf.jabref.external.ExternalFilePanel;
 import net.sf.jabref.external.WriteXMPEntryEditorAction;
 import net.sf.jabref.journals.JournalAbbreviations;
-import net.sf.jabref.gui.date.*;
-import net.sf.jabref.gui.AutoCompleter;
+import net.sf.jabref.gui.*;
+import net.sf.jabref.gui.date.DatePickerButton;
+import net.sf.jabref.imports.BibtexParser;
+import net.sf.jabref.labelPattern.LabelPatternUtil;
+import net.sf.jabref.undo.NamedCompound;
+import net.sf.jabref.undo.UndoableFieldChange;
+import net.sf.jabref.undo.UndoableKeyChange;
+import net.sf.jabref.undo.UndoableRemoveEntry;
 
 /**
- * GUI component that allows editing of the fields of a BibtexEntry. EntryEditor
- * also registers itself as a VetoableChangeListener, receiving events whenever
- * a field of the entry changes, enabling the text fields to update themselves
- * if the change is made from somewhere else.
+ * GUI component that allows editing of the fields of a BibtexEntry (i.e. the
+ * one that shows up, when you double click on an entry in the table)
+ * 
+ * It hosts the tabs (required, general, optional) and the buttons to the left.
+ * 
+ * EntryEditor also registers itself as a VetoableChangeListener, receiving
+ * events whenever a field of the entry changes, enabling the text fields to
+ * update themselves if the change is made from somewhere else.
  */
 public class EntryEditor extends JPanel implements VetoableChangeListener {
 
@@ -118,14 +131,14 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
 
     EntryEditor ths = this;
 
-    HashSet contentSelectors = new HashSet();
+    HashSet<FieldContentSelector> contentSelectors = new HashSet<FieldContentSelector>();
 
     Logger logger = Logger.getLogger(EntryEditor.class.getName());
 
     boolean updateSource = true; // This can be set to false to stop the
                                     // source
 
-    List tabs = new ArrayList();
+    List<Object> tabs = new ArrayList<Object>();
 
     // text area from gettin updated. This is used in cases where the source
     // couldn't be parsed, and the user is given the option to edit it.
@@ -187,7 +200,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         tabs.clear();
         String[] fields = entry.getRequiredFields();
 
-        List fieldList = null;
+        List<String> fieldList = null;
         if (fields != null)
             fieldList = java.util.Arrays.asList(fields);
         reqPan = new EntryEditorTab(frame, panel, fieldList, this, true, Globals.lang("Required fields"));
@@ -236,12 +249,10 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
     private void setupToolBar() {
         tlb = new JToolBar(JToolBar.VERTICAL);
 
-        // tlb.setMargin(new Insets(2,2,2,2));
         tlb.setMargin(new Insets(0, 0, 0, 2));
 
         // The toolbar carries all the key bindings that are valid for the whole
         // window.
-        // tlb.setBackground(GUIGlobals.lightGray);//Color.white);
         ActionMap am = tlb.getActionMap();
         InputMap im = tlb.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
 
@@ -251,13 +262,6 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         am.put("store", storeFieldAction);
         im.put(prefs.getKey("Autogenerate BibTeX keys"), "generateKey");
         am.put("generateKey", generateKeyAction);
-        /*
-         * im.put(prefs.getKey("Entry editor, previous panel"), "left");
-         * im.put(prefs.getKey("Entry editor, previous panel 2"), "left");
-         * am.put("left", switchLeftAction); im.put(prefs.getKey("Entry editor,
-         * next panel"), "right"); im.put(prefs.getKey("Entry editor, next panel
-         * 2"), "right"); am.put("right", switchRightAction);
-         */
         im.put(prefs.getKey("Entry editor, previous entry"), "prev");
         am.put("prev", prevEntryAction);
         im.put(prefs.getKey("Entry editor, next entry"), "next");
@@ -270,25 +274,29 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         am.put("help", helpAction);
 
         tlb.setFloatable(false);
+
+        // Create type-label
+        typeLabel = new TypeLabel(entry.getType().getName());
+        
+        // Add actions (and thus buttons)
         tlb.add(closeAction);
 
-        setLabel();
         tlb.add(typeLabel);
 
-        // tlb.addSeparator();
-        // tlb.add(copyKeyAction);
         tlb.addSeparator();
+        
         tlb.add(generateKeyAction);
+
         tlb.add(writeXmp);
+
         tlb.addSeparator();
 
-        // tlb.add(undoAction);
-        // tlb.add(redoAction);
         tlb.add(deleteAction);
         tlb.add(prevEntryAction);
-
         tlb.add(nextEntryAction);
+        
         tlb.addSeparator();
+        
         tlb.add(helpAction);
 
         Component[] comps = tlb.getComponents();
@@ -299,10 +307,6 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         add(tlb, BorderLayout.WEST);
     }
 
-    private void setLabel() {
-        typeLabel = new TypeLabel(entry.getType().getName());
-    }
-
     /**
      * Rebuild the field tabs. This is called e.g. when a new content selector
      * has been added.
@@ -398,7 +402,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                     if (dir.equals(""))
                         dir = prefs.get(fieldName + Globals.FILETYPE_PREFS_EXT, "");
 
-                    String chosenFile = Globals.getNewFile(frame, new File(dir), "." + fieldName,
+                    String chosenFile = FileDialogs.getNewFile(frame, new File(dir), "." + fieldName,
                         JFileChooser.OPEN_DIALOG, false);
 
                     if (chosenFile != null) {
@@ -491,12 +495,15 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
             StringWriter sw = new StringWriter(200);
 
             try {
-                entry.write(sw, new net.sf.jabref.export.LatexFieldFormatter(), false);
+                LatexFieldFormatter formatter = new LatexFieldFormatter();
+                formatter.setNeverFailOnHashes(true);
+                entry.write(sw, formatter, false);
 
                 String srcString = sw.getBuffer().toString();
                 source.setText(srcString);
                 lastSourceStringAccepted = srcString;
             } catch (IOException ex) {
+                System.out.println("Her");
                 source.setText(ex.getMessage() + "\n\n" + 
                                         Globals.lang("Correct the entry, and "
                     + "reopen editor to display/edit source."));
@@ -542,12 +549,12 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         im.put(Globals.prefs.getKey("Previous tab"), "prevtab");
         am.put("prevtab", frame.prevTab);
         try {
-            HashSet keys = new HashSet(ta
+            HashSet<AWTKeyStroke> keys = new HashSet<AWTKeyStroke>(ta
                 .getFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS));
             keys.clear();
             keys.add(AWTKeyStroke.getAWTKeyStroke("pressed TAB"));
             ta.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, keys);
-            keys = new HashSet(ta
+            keys = new HashSet<AWTKeyStroke>(ta
                 .getFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS));
             keys.clear();
             keys.add(KeyStroke.getKeyStroke("shift pressed TAB"));
@@ -584,7 +591,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
      * Sets the enabled status of all text fields of the entry editor.
      */
     public void setEnabled(boolean enabled) {
-        for (Iterator i = tabs.iterator(); i.hasNext();) {
+        for (Iterator<Object> i = tabs.iterator(); i.hasNext();) {
             Object o = i.next();
             if (o instanceof EntryEditorTab) {
                 ((EntryEditorTab) o).setEnabled(enabled);
@@ -674,7 +681,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         updateAllFields();
         validateAllFields();
         updateSource();
-        panel.showing = be;
+        panel.newEntryShowing(be);
 
     }
 
@@ -707,7 +714,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                 throw new Exception("No entries found.");
 
             NamedCompound compound = new NamedCompound(Globals.lang("source edit"));
-            BibtexEntry nu = db.getEntryById((String) db.getKeySet().iterator().next());
+            BibtexEntry nu = db.getEntryById(db.getKeySet().iterator().next());
             String id = entry.getId();
             String
             // oldKey = entry.getCiteKey(),
@@ -722,32 +729,28 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                 // First, remove fields that the user have removed.
             }
 
-            Object[] fields = entry.getAllFields();
-
-            for (int i = 0; i < fields.length; i++) {
-                if (BibtexFields.isDisplayableField(fields[i].toString())) {
-                    if (nu.getField(fields[i].toString()) == null) {
-                        compound.addEdit(new UndoableFieldChange(entry, fields[i].toString(), entry
-                            .getField(fields[i].toString()), (Object) null));
-                        entry.clearField(fields[i].toString());
+            for (String field : entry.getAllFields()){
+                if (BibtexFields.isDisplayableField(field.toString())) {
+                    if (nu.getField(field.toString()) == null) {
+                        compound.addEdit(new UndoableFieldChange(entry, field.toString(), entry
+                            .getField(field.toString()), null));
+                        entry.clearField(field.toString());
                         anyChanged = true;
                     }
                 }
             }
 
             // Then set all fields that have been set by the user.
-            fields = nu.getAllFields();
-
-            for (int i = 0; i < fields.length; i++) {
-                if (entry.getField(fields[i].toString()) != nu.getField(fields[i].toString())) {
-                    String toSet = (String) nu.getField(fields[i].toString());
+            for (String field : nu.getAllFields()){
+                if (entry.getField(field.toString()) != nu.getField(field.toString())) {
+                    String toSet = nu.getField(field.toString());
 
                     // Test if the field is legally set.
-                    (new LatexFieldFormatter()).format(toSet, fields[i].toString());
+                    (new LatexFieldFormatter()).format(toSet, field.toString());
 
-                    compound.addEdit(new UndoableFieldChange(entry, fields[i].toString(), entry
-                        .getField(fields[i].toString()), toSet));
-                    entry.setField(fields[i].toString(), toSet);
+                    compound.addEdit(new UndoableFieldChange(entry, field.toString(), entry
+                        .getField(field.toString()), toSet));
+                    entry.setField(field.toString(), toSet);
                     anyChanged = true;
                 }
             }
@@ -813,7 +816,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
 
     public void setField(String fieldName, String newFieldData) {
 
-        for (Iterator i = tabs.iterator(); i.hasNext();) {
+        for (Iterator<Object> i = tabs.iterator(); i.hasNext();) {
             Object o = i.next();
             if (o instanceof EntryEditorTab) {
                 ((EntryEditorTab) o).updateField(fieldName, newFieldData);
@@ -826,7 +829,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
      * Sets all the text areas according to the shown entry.
      */
     public void updateAllFields() {
-        for (Iterator i = tabs.iterator(); i.hasNext();) {
+        for (Iterator<Object> i = tabs.iterator(); i.hasNext();) {
             Object o = i.next();
             if (o instanceof EntryEditorTab) {
                 ((EntryEditorTab) o).setEntry(entry);
@@ -838,7 +841,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
      * Removes the "invalid field" color from all text areas.
      */
     public void validateAllFields() {
-        for (Iterator i = tabs.iterator(); i.hasNext();) {
+        for (Iterator<Object> i = tabs.iterator(); i.hasNext();) {
             Object o = i.next();
             if (o instanceof EntryEditorTab) {
                 ((EntryEditorTab) o).validateAllFields();
@@ -848,8 +851,8 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
 
     public void updateAllContentSelectors() {
         if (contentSelectors.size() > 0) {
-            for (Iterator i = contentSelectors.iterator(); i.hasNext();)
-                ((FieldContentSelector) i.next()).rebuildComboBox();
+            for (Iterator<FieldContentSelector> i = contentSelectors.iterator(); i.hasNext();)
+                i.next().rebuildComboBox();
         }
     }
 
@@ -869,9 +872,9 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         storeFieldAction.actionPerformed(new ActionEvent(source, 0, ""));
     }
 
+
     private class TypeLabel extends JPanel {
         private String label;
-
         public TypeLabel(String type) {
             label = type;
             addMouseListener(new MouseAdapter() {
@@ -883,10 +886,8 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                         JPopupMenu typeMenu = new JPopupMenu();
 
                         // typeMenu.addSeparator();
-                        for (Iterator i = BibtexEntryType.ALL_TYPES.keySet().iterator(); i
-                            .hasNext();)
-                            typeMenu.add(new ChangeTypeAction(BibtexEntryType.getType((String) i
-                                .next()), panel));
+                        for (String s: BibtexEntryType.ALL_TYPES.keySet())
+                            typeMenu.add(new ChangeTypeAction(BibtexEntryType.getType(s), panel));
 
                         typeMenu.show(ths, e.getX(), e.getY());
                     }
@@ -898,7 +899,6 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
             Graphics2D g2 = (Graphics2D) g;
             g2.setColor(GUIGlobals.validFieldColor);
             g2.setFont(GUIGlobals.typeNameFont);
-
             FontMetrics fm = g2.getFontMetrics();
             int width = fm.stringWidth(label);
             g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
@@ -992,7 +992,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         }
 
         public void actionPerformed(ActionEvent e) {
-            String s = (String) (entry.getField(BibtexFields.KEY_FIELD));
+            String s = (entry.getField(BibtexFields.KEY_FIELD));
             StringSelection ss = new StringSelection(s);
 
             if (s != null)
@@ -1058,6 +1058,8 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                     // fe.setLabelColor(GUIGlobals.nullFieldColor);
                     fe.setBackground(GUIGlobals.validFieldBackground);
 
+                if (fe.getTextComponent().hasFocus())
+                    fe.setBackground(GUIGlobals.activeEditor);
                 updateSource();
                 panel.markBaseChanged();
             }
@@ -1097,7 +1099,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                         if (toSet != null)
                             (new LatexFieldFormatter()).format(toSet, fe.getFieldName());
 
-                        Object oldValue = entry.getField(fe.getFieldName());
+                        String oldValue = entry.getField(fe.getFieldName());
 
                         if (toSet != null)
                             entry.setField(fe.getFieldName(), toSet);
@@ -1114,7 +1116,7 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                         // See if we need to update an AutoCompleter instance:
                         AutoCompleter aComp = panel.getAutoCompleter(fe.getFieldName());
                         if (aComp != null)
-                            aComp.addAll(toSet);
+                            aComp.addAll(toSet, entry);
 
                         // Add an UndoableFieldChange to the baseframe's
                         // undoManager.
@@ -1123,14 +1125,6 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                         updateSource();
                         panel.markBaseChanged();
 
-                        // TODO: is this a safe solution to keep selection on
-                        // entry?
-                        SwingUtilities.invokeLater(new Runnable() {
-                            public void run() {
-                                panel.highlightEntry(entry);
-                            }
-                        });
-
                     } catch (IllegalArgumentException ex) {
                         JOptionPane.showMessageDialog(frame, Globals.lang("Error") + ": " + ex.getMessage(), Globals
                             .lang("Error setting field"), JOptionPane.ERROR_MESSAGE);
@@ -1141,6 +1135,8 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
                     // We set the field and label color.
                     fe.setBackground(GUIGlobals.validFieldBackground);
                 }
+                if (fe.getTextComponent().hasFocus())
+                    fe.setBackground(GUIGlobals.activeEditor);
             } else if ((source.isEditable())
                 && (!source.getText().equals(lastSourceStringAccepted))) {
                 boolean accepted = storeSource(true);
@@ -1264,10 +1260,10 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
 
                 // Store undo information:
                 panel.undoManager.addEdit(new UndoableKeyChange(panel.database, entry.getId(),
-                    (String) oldValue, (String) entry.getField(BibtexFields.KEY_FIELD)));
+                    (String) oldValue, entry.getField(BibtexFields.KEY_FIELD)));
 
                 // here we update the field
-                String bibtexKeyData = (String) entry.getField(BibtexFields.KEY_FIELD);
+                String bibtexKeyData = entry.getField(BibtexFields.KEY_FIELD);
 
                 // set the field named for "bibtexkey"
                 setField(BibtexFields.KEY_FIELD, bibtexKeyData);
@@ -1367,22 +1363,6 @@ public class EntryEditor extends JPanel implements VetoableChangeListener {
         }
     }
 
-    /**
-     * Scans all groups.
-     * 
-     * @return true if the specified entry is contained in any ExplicitGroup,
-     *         false otherwise.
-     */
-    private boolean containedInExplicitGroup(BibtexEntry entry) {
-        AbstractGroup[] matchingGroups = panel.getGroupSelector().getGroupTreeRoot()
-            .getMatchingGroups(entry);
-        for (int i = 0; i < matchingGroups.length; ++i) {
-            if (matchingGroups[i] instanceof ExplicitGroup)
-                return true;
-        }
-        return false;
-    }
-
     private void warnDuplicateBibtexkey() {
         panel.output(Globals.lang("Warning") + ": " + Globals.lang("Duplicate BibTeX key."));
 
