@@ -35,13 +35,18 @@
 
 package net.sf.jabref ;
 
-import java.awt.* ;
-import java.awt.event.*;
-import javax.swing.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.MouseEvent;
+import java.awt.event.MouseListener;
+import java.net.URL;
+
+import javax.swing.AbstractAction;
+import javax.swing.ImageIcon;
+import javax.swing.JPopupMenu;
+import javax.swing.KeyStroke;
 import javax.swing.text.JTextComponent;
-import java.net.*;
-import java.awt.datatransfer.*;
-import net.sf.jabref.util.*;
+
+import net.sf.jabref.util.CaseChangeMenu;
 
 public class FieldTextMenu implements MouseListener
 {
@@ -160,12 +165,8 @@ public class FieldTextMenu implements MouseListener
 
     public void actionPerformed(ActionEvent e)
     {
-      Clipboard systemClip = Toolkit.getDefaultToolkit().getSystemClipboard();
-
       try
       {
-//        String data = ( String ) systemClip.getContents( null ).getTransferData(
-//            DataFlavor.stringFlavor ) ;
         String data = ClipBoardManager.clipBoard.getClipboardContents() ;
         if (data != null)
           if (data.length() > 0)
