@@ -24,39 +24,46 @@
  */
 package net.sf.jabref;
 
+import com.jgoodies.looks.FontPolicies;
+import com.jgoodies.looks.FontPolicy;
+import com.jgoodies.looks.FontSet;
+import com.jgoodies.looks.FontSets;
+import com.jgoodies.looks.plastic.Plastic3DLookAndFeel;
+import com.jgoodies.looks.windows.WindowsLookAndFeel;
+import gnu.dtools.ritopt.BooleanOption;
+import gnu.dtools.ritopt.Options;
+import gnu.dtools.ritopt.StringOption;
 import net.sf.jabref.export.*;
 import net.sf.jabref.imports.*;
-import net.sf.jabref.wizard.auximport.*;
+import net.sf.jabref.plugin.PluginCore;
+import net.sf.jabref.plugin.SidePanePlugin;
+import net.sf.jabref.plugin.PluginInstaller;
+import net.sf.jabref.plugin.core.JabRefPlugin;
+import net.sf.jabref.plugin.core.generated._JabRefPlugin;
+import net.sf.jabref.plugin.core.generated._JabRefPlugin.EntryFetcherExtension;
 import net.sf.jabref.remote.RemoteListener;
-
-import gnu.dtools.ritopt.*;
-import java.awt.*;
-import java.awt.event.KeyEvent;
-
-import java.io.*;
-import java.io.File;
-
-import java.util.*;
-import java.util.List;
-import java.lang.reflect.Method;
-import java.lang.reflect.InvocationTargetException;
+import net.sf.jabref.util.Pair;
+import net.sf.jabref.wizard.auximport.AuxCommandLine;
 
 import javax.swing.*;
 import javax.swing.plaf.metal.MetalLookAndFeel;
+import java.awt.*;
+import java.awt.event.KeyEvent;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.util.*;
+import java.util.List;
 
-import com.jgoodies.looks.plastic.Plastic3DLookAndFeel;
-import com.jgoodies.looks.plastic.PlasticLookAndFeel;
-import com.jgoodies.looks.windows.WindowsLookAndFeel;
-import com.jgoodies.looks.FontSet;
-import com.jgoodies.looks.FontSets;
-import com.jgoodies.looks.FontPolicy;
-import com.jgoodies.looks.FontPolicies;
+import spin.Spin;
 
-//import javax.swing.UIManager;
-//import javax.swing.UIDefaults;
-//import javax.swing.UnsupportedLookAndFeelException;
+/**
+ * JabRef Main Class - The application gets started here.
+ *
+ */
 public class JabRef {
-    public static JabRef ths;
+
+	public static JabRef singleton;
     public static RemoteListener remoteListener = null;
     public JabRefFrame jrf;
     public Options options;
@@ -64,85 +71,107 @@ public class JabRef {
 
     boolean graphicFailure = false;
 
-    StringOption importFile, exportFile, exportPrefs, importPrefs, auxImExport, importToOpenBase;
+    StringOption importFile, exportFile, exportPrefs, importPrefs, auxImExport, importToOpenBase, fetcherEngine;
     BooleanOption helpO, disableGui, blank, loadSess, showVersion, disableSplash;
-    /*
-    * class StringArrayOption extends ArrayOption { public public void
-    * modify(String value) { } public void modify(String[] value) { } public
-    * Object[] getObjectArray() { return null; } public String getTypeName() {
-    * return "Strings"; } public String getStringValue() { return ""; } public
-    * Object getObject() { return null; } }
-    */
+
+    public static final int MAX_DIALOG_WARNINGS = 10;
+
     public static void main(String[] args) {
         new JabRef(args);
     }
 
-    public JabRef(String[] args) {
+    protected JabRef(String[] args) {
 
-        ths = this;
-	// Activate antialiasing on Java 5:
-	System.getProperties().put( "swing.aatext", "true" );
-        // The following two lines signal that the system proxy settings should be used:
-        System.setProperty("java.net.useSystemProxies", "true");
-        System.getProperties().put( "proxySet", "true" );
+		singleton = this;
 
-        JabRefPreferences prefs = JabRefPreferences.getInstance();
-        Globals.prefs = prefs;
-        Globals.setLanguage(prefs.get("language"), "");
-                
-        Globals.importFormatReader.resetImportFormats();
-        BibtexEntryType.loadCustomEntryTypes(prefs);
-        // Build the list of available export formats:
-        ExportFormats.initAllExports();
+		// The following two lines signal that the system proxy settings should
+		// be used:
+		System.setProperty("java.net.useSystemProxies", "true");
+		System.getProperties().put("proxySet", "true");
 
-        // Read list(s) of journal names and abbreviations:
-        //Globals.turnOnFileLogging();
+		JabRefPreferences prefs = JabRefPreferences.getInstance();
 
+        // See if there are plugins scheduled for deletion:
+        if (prefs.hasKey("deletePlugins") && (prefs.get("deletePlugins").length() > 0)) {
+            String[] toDelete = prefs.getStringArray("deletePlugins");
+            PluginInstaller.deletePluginsOnStartup(toDelete);
+            prefs.put("deletePlugins", "");
+        }
+
+        Globals.startBackgroundTasks();
+		Globals.prefs = prefs;
+		Globals.setLanguage(prefs.get("language"), "");
+
+		/*
+		 * The Plug-in System is started automatically on the first call to
+		 * PluginCore.getManager().
+		 * 
+		 * Plug-ins are activated on the first call to their getInstance method.
+		 */
+
+        // Update which fields should be treated as numeric, based on preferences:
+        BibtexFields.setNumericFieldsFromPrefs();
+		
+		/* Build list of Import and Export formats */
+		Globals.importFormatReader.resetImportFormats();
+		BibtexEntryType.loadCustomEntryTypes(prefs);
+		ExportFormats.initAllExports();
+		
+		// Read list(s) of journal names and abbreviations:
         Globals.initializeJournalNames();
 
-        if (Globals.prefs.getBoolean("useRemoteServer")) {
-            remoteListener = RemoteListener.openRemoteListener(this);
-            if (remoteListener != null) {
-                remoteListener.start();
-            }
+		// Check for running JabRef
+		if (Globals.prefs.getBoolean("useRemoteServer")) {
+			remoteListener = RemoteListener.openRemoteListener(this);
 
-        // Unless we are alone, try to contact already running JabRef:
-	    if (remoteListener == null) {
-		    if (RemoteListener.sendToActiveJabRefInstance(args)) {
-                // We have successfully sent our command line options through the socket to
-                // another JabRef instance. So we assume it's all taken care of, and quit.
-                System.out.println(Globals.lang("Arguments passed on to running JabRef instance. Shutting down."));
-                System.exit(0);
-            }
-        }
+			if (remoteListener == null) {
+				// Unless we are alone, try to contact already running JabRef:
+				if (RemoteListener.sendToActiveJabRefInstance(args)) {
+
+					/*
+					 * We have successfully sent our command line options
+					 * through the socket to another JabRef instance. So we
+					 * assume it's all taken care of, and quit.
+					 */
+					System.out
+							.println(Globals
+									.lang("Arguments passed on to running JabRef instance. Shutting down."));
+					System.exit(0);
+				}
+			} else {
+				// No listener found, thus we are the first instance to be
+				// started.
+				remoteListener.start();
+			}
+		}
+
+		/*
+		 * See if the user has a personal journal list set up. If so, add these
+		 * journal names and abbreviations to the list:
+		 */
+		String personalJournalList = prefs.get("personalJournalList");
+		if (personalJournalList != null) {
+			try {
+				Globals.journalAbbrev.readJournalList(new File(
+						personalJournalList));
+			} catch (FileNotFoundException e) {
+				e.printStackTrace();
+			}
+		}
+
+		/*
+		 * Make sure of a proper cleanup when quitting (e.g. deleting temporary
+		 * files).
+		 * 
+		 * CO 2007-07-12: Since this is deprecated, commented out:
+		 * 
+		 * System.runFinalizersOnExit(true);
+		 * 
+		 */
+		
+		openWindow(processArguments(args, true));
 	}
 
-      /**
-       * See if the user has a personal journal list set up. If so, add these
-       * journal names and abbreviations to the list:
-       */
-      String personalJournalList = prefs.get("personalJournalList");
-      if (personalJournalList != null) {
-          try {
-              Globals.journalAbbrev.readJournalList(new File(personalJournalList));
-          } catch (FileNotFoundException e) {
-              e.printStackTrace();
-          }
-      }
-
-        
-        //System.setProperty("sun.awt.noerasebackground", "true");
-        
-        //System.out.println(java.awt.Toolkit.getDefaultToolkit().getDesktopProperty("awt.dynamicLayoutSupported"));
-        // Make sure of a proper cleanup when quitting (e.g. deleting temporary
-        // files).
-        System.runFinalizersOnExit(true);
-
-        Vector loaded = processArguments(args, true);
-        openWindow(loaded);
-        //System.out.println("1");
-    }
-
     private void setupOptions() {
 
         importFile = new StringOption("");
@@ -157,6 +186,7 @@ public class JabRef {
         importPrefs = new StringOption("jabref_prefs.xml");
         auxImExport = new StringOption("");
         importToOpenBase = new StringOption("");
+        fetcherEngine = new StringOption("");
 
         options = new Options("JabRef "); // Create an options repository.
         options.setVersion(GUIGlobals.version);
@@ -188,18 +218,16 @@ public class JabRef {
 
         options.register("importToOpen", '\0', Globals.lang("Import to open tab"), importToOpenBase);
 
+        options.register("fetch", 'f', Globals.lang("Run Fetcher, e.g. \"--fetch=Medline:cancer\""), fetcherEngine);
+
         options.setUseMenu(false);
     }
 
-    public Vector processArguments(String[] args, boolean initialStartup) {
+    public Vector<ParserResult> processArguments(String[] args, boolean initialStartup) {
 
         setupOptions();
         String[] leftOver = options.process(args);
 
-
-
-        //Util.pr(": Options processed");
-
         if (initialStartup && showVersion.isInvoked()) {
             options.displayVersion();
             disableGui.setInvoked(true);
@@ -218,16 +246,16 @@ public class JabRef {
                 + ".");
             System.exit(0);
         }
-
+        
+        boolean commandmode = disableGui.isInvoked() || fetcherEngine.isInvoked();
+        
         // First we quickly scan the command line parameters for any that signal
         // that the GUI
         // should not be opened. This is used to decide whether we should show the
         // splash screen or not.
-        if (initialStartup && !disableGui.isInvoked() && !disableSplash.isInvoked()) {
+        if (initialStartup && !commandmode && !disableSplash.isInvoked()) {
             try {
-
                 splashScreen = SplashScreen.splash();
-
             } catch (Throwable ex) {
                 graphicFailure = true;
                 System.err.println(Globals.lang("Unable to create graphical interface")
@@ -235,10 +263,9 @@ public class JabRef {
             }
         }
 
-        //Util.pr("JabRef "+GUIGlobals.version);
         // Vector to put imported/loaded database(s) in.
-        Vector loaded = new Vector();
-        Vector toImport = new Vector();
+        Vector<ParserResult> loaded = new Vector<ParserResult>();
+        Vector<String> toImport = new Vector<String>();
         if (!blank.isInvoked() && (leftOver.length > 0))  {
             for (int i = 0; i < leftOver.length; i++) {
                 // Leftover arguments that have a "bib" extension are interpreted as
@@ -247,7 +274,7 @@ public class JabRef {
                 boolean bibExtension = leftOver[i].toLowerCase().endsWith("bib");
                 ParserResult pr = null;
                 if (bibExtension)
-                    pr = openBibFile(leftOver[i]);
+                    pr = openBibFile(leftOver[i], false);
 
                 if ((pr == null) || (pr == ParserResult.INVALID_FORMAT)) {
                     // We will try to import this file. Normally we
@@ -262,6 +289,8 @@ public class JabRef {
                         ParserResult res = importToOpenBase(leftOver[i]);
                         if (res != null)
                             loaded.add(res);
+                        else
+                            loaded.add(ParserResult.INVALID_FORMAT);
                     }
                 }
                 else
@@ -270,68 +299,15 @@ public class JabRef {
             }
         }
 
-        //Util.pr(": Checked blank");
-
         if (!blank.isInvoked() && importFile.isInvoked()) {
             toImport.add(importFile.getStringValue());
         }
 
-        if (toImport.size() > 0) for (int i = 0; i < toImport.size(); i++) {
-            String[] data = ((String) toImport.elementAt(i)).split(",");
-
-            try {
-
-                if ((data.length > 1) && !"*".equals(data[1])) {
-                    System.out.println(Globals.lang("Importing") + ": " + data[0]);
-                    List entries =
-                            Globals.importFormatReader.importFromFile(data[1],
-                                    data[0].replaceAll("~", System.getProperty("user.home")));
-                    BibtexDatabase base = ImportFormatReader.createDatabase(entries);
-                    ParserResult pr = new ParserResult(base, null, new HashMap());
-                    loaded.add(pr);
-
-                } else {
-                    // * means "guess the format":
-                    System.out.println(Globals.lang("Importing in unknown format")
-                            + ": " + data[0]);
-
-                    Object[] o =
-                            Globals.importFormatReader.importUnknownFormat(data[0]
-                                    .replaceAll("~", System.getProperty("user.home")));
-                    String formatName = (String) o[0];
-
-                    if (formatName == null) {
-                        System.err.println(Globals.lang("Error opening file") + " '" + data[0] + "'");
-                    } else if (formatName.equals(ImportFormatReader.BIBTEX_FORMAT)) {
-                        ParserResult pr = (ParserResult) o[1];
-                        loaded.add(pr);
-
-                    } else {
-                        List entries = (java.util.List) o[1];
-                        if (entries != null)
-                            System.out.println(Globals.lang("Format used") + ": "
-                                    + formatName);
-                        else
-                            System.out.println(Globals.lang(
-                                    "Could not find a suitable import format."));
-
-                        if (entries != null) {
-                            BibtexDatabase base = ImportFormatReader.createDatabase(entries);
-                            ParserResult pr = new ParserResult(base, null, new HashMap());
-
-                            //pr.setFile(new File(data[0]));
-                            loaded.add(pr);
-                        }
-                    }
-                }
-            } catch (IOException ex) {
-                System.err.println(Globals.lang("Error opening file") + " '"
-                        + data[0] + "': " + ex.getMessage());
-            }
-
-
-        }
-
+        for (String filenameString : toImport) {
+			ParserResult pr = importFile(filenameString);
+			if (pr != null)
+				loaded.add(pr);
+		}
 
         if (!blank.isInvoked() && importToOpenBase.isInvoked()) {
             ParserResult res = importToOpenBase(importToOpenBase.getStringValue());
@@ -339,6 +315,12 @@ public class JabRef {
                 loaded.add(res);
         }
 
+        if (!blank.isInvoked() && fetcherEngine.isInvoked()) {
+            ParserResult res = fetch(fetcherEngine.getStringValue());
+            if (res != null)
+                loaded.add(res);
+        }
+
         if (exportFile.isInvoked()) {
             if (loaded.size() > 0) {
                 String[] data = exportFile.getStringValue().split(",");
@@ -348,13 +330,13 @@ public class JabRef {
                     // format to the given file.
                     if (loaded.size() > 0) {
                         ParserResult pr =
-                            (ParserResult) loaded.elementAt(loaded.size() - 1);
+                            loaded.elementAt(loaded.size() - 1);
 
                         try {
                             System.out.println(Globals.lang("Saving") + ": " + data[0]);
                             SaveSession session = FileActions.saveDatabase(pr.getDatabase(),
                                 new MetaData(pr.getMetaData(),pr.getDatabase()), new File(data[0]), Globals.prefs,
-                                false, false, Globals.prefs.get("defaultEncoding"));
+                                false, false, Globals.prefs.get("defaultEncoding"), false);
                             // Show just a warning message if encoding didn't work for all characters:
                             if (!session.getWriter().couldEncodeAll())
                                 System.err.println(Globals.lang("Warning")+": "+
@@ -371,7 +353,7 @@ public class JabRef {
                 } else if (data.length == 2) {
                     // This signals that the latest import should be stored in the given
                     // format to the given file.
-                    ParserResult pr = (ParserResult) loaded.elementAt(loaded.size() - 1);
+                    ParserResult pr = loaded.elementAt(loaded.size() - 1);
 
                     // Set the global variable for this database's file directory before exporting,
                     // so formatters can resolve linked files correctly.
@@ -380,11 +362,13 @@ public class JabRef {
                     metaData.setFile(pr.getFile());
                     Globals.prefs.fileDirForDatabase = metaData.getFileDirectory(GUIGlobals.FILE_FIELD);
                     System.out.println(Globals.lang("Exporting") + ": " + data[0]);
-                    ExportFormat format = ExportFormats.getExportFormat(data[1]);
+                    IExportFormat format = ExportFormats.getExportFormat(data[1]);
                     if (format != null) {
                         // We have an ExportFormat instance:
                         try {
-                            format.performExport(pr.getDatabase(), data[0], pr.getEncoding(), null);
+                            format.performExport(pr.getDatabase(), 
+                                    new MetaData(pr.getMetaData(), pr.getDatabase()),
+                                    data[0], pr.getEncoding(), null);
                         } catch (Exception ex) {
                             System.err.println(Globals.lang("Could not export file")
                                 + " '" + data[0] + "': " + ex.getMessage());
@@ -429,7 +413,7 @@ public class JabRef {
                 String[] data = auxImExport.getStringValue().split(",");
 
                 if (data.length == 2) {
-                    ParserResult pr = (ParserResult) loaded.firstElement();
+                    ParserResult pr = loaded.firstElement();
                     AuxCommandLine acl = new AuxCommandLine(data[0], pr.getDatabase());
                     BibtexDatabase newBase = acl.perform();
 
@@ -445,7 +429,7 @@ public class JabRef {
                                     + subName);
                                 SaveSession session = FileActions.saveDatabase(newBase, new MetaData(), // no Metadata
                                     new File(subName), Globals.prefs, false, false,
-                                    Globals.prefs.get("defaultEncoding"));
+                                    Globals.prefs.get("defaultEncoding"), false);
                                 // Show just a warning message if encoding didn't work for all characters:
                                 if (!session.getWriter().couldEncodeAll())
                                     System.err.println(Globals.lang("Warning")+": "+
@@ -479,38 +463,66 @@ public class JabRef {
         return loaded;
     }
 
-    ParserResult importFiletypeUnknown(String fname) {
-        Object[] o =
-                Globals.importFormatReader.importUnknownFormat(fname
-                        .replaceAll("~", System.getProperty("user.home")));
-        String formatName = (String) o[0];
+    /**
+     * Run an entry fetcher from the command line.
+     * 
+     * Note that this only works headlessly if the EntryFetcher does not show
+     * any GUI.
+     * 
+     * @param fetchCommand
+     *            A string containing both the fetcher to use (id of
+     *            EntryFetcherExtension minus Fetcher) and the search query,
+     *            separated by a :
+     * @return A parser result containing the entries fetched or null if an
+     *         error occurred.
+     */
+    protected ParserResult fetch(String fetchCommand) {
 
-        if (formatName == null) {
-            System.err.println(Globals.lang("Error opening file") + " '" + fname + "'");
-        } else if (formatName.equals(ImportFormatReader.BIBTEX_FORMAT)) {
-            ParserResult pr = (ParserResult) o[1];
-            return pr;
-        } else {
-            List entries = (java.util.List) o[1];
-            if (entries != null)
-                System.out.println(Globals.lang("Format used") + ": "
-                        + formatName);
-            else
-                System.out.println(Globals.lang(
-                        "Could not find a suitable import format."));
-
-            if (entries != null) {
-                BibtexDatabase base = ImportFormatReader.createDatabase(entries);
-                ParserResult pr = new ParserResult(base, null, new HashMap());
-                return pr;
+        if (fetchCommand == null || !fetchCommand.contains(":") ||
+            fetchCommand.split(":").length != 2) {
+            System.out.println(Globals.lang("Expected syntax for --fetch='<name of fetcher>:<query>'"));
+            System.out.println(Globals.lang("The following fetchers are available:"));
+            for (EntryFetcherExtension e : JabRefPlugin.getInstance(PluginCore.getManager())
+                .getEntryFetcherExtensions()) {
+                System.out.println("  " + e.getId().replaceAll("Fetcher", "").toLowerCase());
             }
+            return null;
         }
-        
-        // not reached
-        return null;
+
+        String engine = fetchCommand.split(":")[0];
+        String query = fetchCommand.split(":")[1];
+
+        EntryFetcher fetcher = null;
+        for (EntryFetcherExtension e : JabRefPlugin.getInstance(PluginCore.getManager())
+            .getEntryFetcherExtensions()) {
+            if (engine.toLowerCase().equals(e.getId().replaceAll("Fetcher", "").toLowerCase()))
+                fetcher = e.getEntryFetcher();
+        }
+
+        if (fetcher == null) {
+            System.out.println(Globals.lang("Could not find fetcher '%0'", engine));
+            System.out.println(Globals.lang("The following fetchers are available:"));
+            for (EntryFetcherExtension e : JabRefPlugin.getInstance(PluginCore.getManager())
+                .getEntryFetcherExtensions()) {
+                System.out.println("  " + e.getId().replaceAll("Fetcher", "").toLowerCase());
+            }
+            return null;
+        }
+
+        System.out.println(Globals.lang("Running Query '%0' with fetcher '%1'.", query, engine) +
+            " " + Globals.lang("Please wait!"));
+        Collection<BibtexEntry> result = new ImportInspectionCommandLine().query(query, fetcher);
+
+        if (result == null || result.size() == 0) {
+            System.out.println(Globals.lang(
+                "Query '%0' with fetcher '%1' did not return any results.", query, engine));
+            return null;
+        }
+
+        return new ParserResult(result);
     }
 
-    public void openWindow(Vector loaded) {
+	public void openWindow(Vector<ParserResult> loaded) {
         if (!graphicFailure && !disableGui.isInvoked()) {
             // Call the method performCompatibilityUpdate(), which does any
             // necessary changes for users with a preference set from an older
@@ -663,29 +675,28 @@ public class JabRef {
             if (!blank.isInvoked() && Globals.prefs.getBoolean("openLastEdited") && (Globals.prefs.get("lastEdited") != null)) {
                 // How to handle errors in the databases to open?
                 String[] names = Globals.prefs.getStringArray("lastEdited");
-lastEdLoop: 
+                lastEdLoop: 
                 for (int i = 0; i < names.length; i++) {
                     File fileToOpen = new File(names[i]);
 
                     for (int j = 0; j < loaded.size(); j++) {
-                        ParserResult pr = (ParserResult) loaded.elementAt(j);
+                        ParserResult pr = loaded.elementAt(j);
 
                         if ((pr.getFile() != null) &&pr.getFile().equals(fileToOpen))
                             continue lastEdLoop;
                     }
 
                     if (fileToOpen.exists()) {
-                        ParserResult pr = openBibFile(names[i]);
+                        ParserResult pr = openBibFile(names[i], false);
 
                         if (pr != null) {
 
-			    if (pr == ParserResult.INVALID_FORMAT) {
-				System.out.println(Globals.lang("Error opening file")+" '"+fileToOpen.getPath()+"'");
-			    }
-			    else
-				loaded.add(pr);
+                            if (pr.isInvalid()) {
+                                System.out.println(Globals.lang("Error opening file")+" '"+fileToOpen.getPath()+"'");
+                            }
+                            loaded.add(pr);
 
-			}
+                        }
                     }
                 }
             }
@@ -699,13 +710,28 @@ lastEdLoop:
             jrf = new JabRefFrame();
 
             // Add all loaded databases to the frame:
-	    boolean first = true;
+            
+	        boolean first = true;
+            List<File> postponed = new ArrayList<File>();
+            List<ParserResult> failed = new ArrayList<ParserResult>();
             if (loaded.size() > 0) {
-                for (Iterator i=loaded.iterator(); i.hasNext();) {
-                    ParserResult pr = (ParserResult)i.next();
-		            BasePanel panel = jrf.addTab(pr.getDatabase(), pr.getFile(),
-                            pr.getMetaData(), pr.getEncoding(), first);
-                    first = false;
+                for (Iterator<ParserResult> i = loaded.iterator(); i.hasNext();){
+                    ParserResult pr = i.next();
+                    if (pr.isInvalid()) {
+
+                        failed.add(pr);
+                        i.remove();
+                    }
+                    else if (!pr.isPostponedAutosaveFound()) {
+                        jrf.addTab(pr.getDatabase(), pr.getFile(),
+                                pr.getMetaData(), pr.getEncoding(), first);
+                        first = false;
+                    }
+                    else {
+                        i.remove();
+                        postponed.add(pr.getFile());
+
+                    }
                 }
             }
 
@@ -722,20 +748,45 @@ lastEdLoop:
                 +"is an early beta version. Do not use it without backing up your files!"),
                     Globals.lang("Beta version"), JOptionPane.WARNING_MESSAGE);*/
 
+
+            // Start auto save timer:
+            if (Globals.prefs.getBoolean("autoSave"))
+                Globals.startAutoSaveManager(jrf);
+
             //Util.pr(": Showing frame");
             jrf.setVisible(true);
+            // If we are set to remember the window location, we also remember the maximised
+            // state. This needs to be set after the window has been made visible, so we
+            // do it here:
+             if (Globals.prefs.getBoolean("rememberWindowLocation") &&
+                     Globals.prefs.getBoolean("windowMaximised")) {
+                 jrf.setExtendedState(JFrame.MAXIMIZED_BOTH);
+             }
+
 
             // TEST TEST TEST TEST TEST TEST
-            startOOPlugin(jrf);
+            startSidePanePlugins(jrf);
+
+            for (ParserResult pr : failed) {
+                String message = "<html>"+Globals.lang("Error opening file '%0'.", pr.getFile().getName())
+                    +"<p>"+pr.getErrorMessage()+"</html>";
+
+                JOptionPane.showMessageDialog(jrf, message, Globals.lang("Error opening file"),
+                    JOptionPane.ERROR_MESSAGE);
+            }
 
             for (int i = 0; i < loaded.size(); i++) {
-                ParserResult pr = (ParserResult) loaded.elementAt(i);
+                ParserResult pr = loaded.elementAt(i);
                 if (Globals.prefs.getBoolean("displayKeyWarningDialogAtStartup") && pr.hasWarnings()) {
                     String[] wrns = pr.warnings();
-                    StringBuffer wrn = new StringBuffer();
-                    for (int j = 0; j<wrns.length; j++)
+                    StringBuilder wrn = new StringBuilder();
+                    for (int j = 0; j<Math.min(MAX_DIALOG_WARNINGS, wrns.length); j++)
                         wrn.append(j + 1).append(". ").append(wrns[j]).append("\n");
-                    if (wrn.length() > 0)
+                    if (wrns.length > MAX_DIALOG_WARNINGS) {
+                        wrn.append("... ");
+                        wrn.append(Globals.lang("%0 warnings", String.valueOf(wrns.length)));
+                    }
+                    else if (wrn.length() > 0)
                         wrn.deleteCharAt(wrn.length() - 1);
                     jrf.showBaseAt(i);
                     JOptionPane.showMessageDialog(jrf, wrn.toString(),
@@ -750,13 +801,20 @@ lastEdLoop:
             // if the database contents should be modified due to new features
             // in this version of JabRef:
             for (int i = 0; i < loaded.size(); i++) {
-                ParserResult pr = (ParserResult) loaded.elementAt(i);
+                ParserResult pr = loaded.elementAt(i);
                 BasePanel panel = jrf.baseAt(i);
                 OpenDatabaseAction.performPostOpenActions(panel, pr, true);
             }
 
             //Util.pr(": Finished adding panels");
 
+            // If any database loading was postponed due to an autosave, schedule them
+            // for handing now:
+            if (postponed.size() > 0) {
+                AutosaveStartupPrompter asp = new AutosaveStartupPrompter(jrf, postponed);
+                SwingUtilities.invokeLater(asp);
+            }
+
             if (loaded.size() > 0) {
                 jrf.tabbedPane.setSelectedIndex(0);
                 new FocusRequester(((BasePanel) jrf.tabbedPane.getComponentAt(0)).mainTable);
@@ -766,42 +824,49 @@ lastEdLoop:
     }
 
     /**
-     * Morten Alver (sept. 06): I'm adding this to run the functionality for
-     * OpenOffice integration. I'm not sure yet how to handle deployment issues,
-     * because the OO stuff requires several OO jars on the classpath, which we
-     * shouldn't distribute because OO installations already have them.
-     *
-     * To avoid introducing dependencies on these jars for the time being,
-     * I'm keeping OO "plugin" files separate, only invoking them through reflection.
+     * Go through all registered instances of SidePanePlugin, and register them
+     * in the SidePaneManager.
      *
      * @param jrf The JabRefFrame.
      */
-    private void startOOPlugin(JabRefFrame jrf) {
+    private void startSidePanePlugins(JabRefFrame jrf) {
 
-
-        try {
-            Class c = Class.forName("net.sf.jabref.oo.OOTestPanel");
-            Method m = c.getDeclaredMethod("open",
-                    new Class[] {JabRefFrame.class});
-            Object i = c.newInstance();
-            Object r = m.invoke(i, new Object[] {jrf});
-        } catch (Exception e) {
-            // Do nothing.
-            //System.out.println("OO plugin not found.");
-            //e.printStackTrace();
+        JabRefPlugin jabrefPlugin = JabRefPlugin.getInstance(PluginCore.getManager());
+        List<_JabRefPlugin.SidePanePluginExtension> plugins = jabrefPlugin.getSidePanePluginExtensions();
+        for (_JabRefPlugin.SidePanePluginExtension extension : plugins) {
+            SidePanePlugin plugin = extension.getSidePanePlugin();
+            plugin.init(jrf, jrf.sidePaneManager);
+            SidePaneComponent comp = plugin.getSidePaneComponent();
+            jrf.sidePaneManager.register(comp.getName(), comp);
+            jrf.addPluginMenuItem(plugin.getMenuItem());
         }
-        
     }
 
-    public static ParserResult openBibFile(String name) {
+    public static ParserResult openBibFile(String name, boolean ignoreAutosave) {
         System.out.println(Globals.lang("Opening") + ": " + name);
-
+        File file = new File(name);
         try {
-            File file = new File(name);
+
+            if (!ignoreAutosave) {
+                boolean autoSaveFound = AutoSaveManager.newerAutoSaveExists(file);
+                if (autoSaveFound) {
+                    // We have found a newer autosave. Make a note of this, so it can be
+                    // handled after startup:
+                    ParserResult postp = new ParserResult(null, null, null);
+                    postp.setPostponedAutosaveFound(true);
+                    postp.setFile(file);
+                    return postp;
+                }
+            }
             String encoding = Globals.prefs.get("defaultEncoding");
             ParserResult pr = OpenDatabaseAction.loadDatabase(file, encoding);
-            if (pr == null)
-                return ParserResult.INVALID_FORMAT;
+            if (pr == null) {
+                pr = new ParserResult(null, null, null);
+                pr.setFile(file);
+                pr.setInvalid(true);
+                return pr;
+
+            }
             pr.setFile(file);
             if (pr.hasWarnings()) {
                 String[] warn = pr.warnings();
@@ -811,70 +876,61 @@ lastEdLoop:
             }
             return pr;
         } catch (Throwable ex) {
-            //System.err.println(Globals.lang("Error opening file")+" '"+ name+"':
-            // "+ex.getMessage());
-            System.err.println(Globals.lang("Error opening file") + ": "
-                + ex.getMessage());
+            ParserResult pr = new ParserResult(null, null, null);
+            pr.setFile(file);
+            pr.setInvalid(true);
+            pr.setErrorMessage(ex.getMessage());
+            return pr;
         }
 
-        return null;
     }
 
-    public static ParserResult importToOpenBase(String argument) {
-        String[] data = argument.split(",");
+    public static ParserResult importFile(String argument){
+    	String[] data = argument.split(",");
         try {
             if ((data.length > 1) && !"*".equals(data[1])) {
                 System.out.println(Globals.lang("Importing") + ": " + data[0]);
-                List entries =
-                    Globals.importFormatReader.importFromFile(data[1],
-                        data[0].replaceAll("~", System.getProperty("user.home")));
-                BibtexDatabase base = ImportFormatReader.createDatabase(entries);
-                ParserResult pr = new ParserResult(base, null, new HashMap());
-                pr.setToOpenTab(true);
-                //loaded.add(pr);
-                return pr;
+                List<BibtexEntry> entries =
+                        Globals.importFormatReader.importFromFile(data[1],
+                                data[0].replaceAll("~", System.getProperty("user.home")));
+                return new ParserResult(entries);
             } else {
                 // * means "guess the format":
                 System.out.println(Globals.lang("Importing in unknown format")
-                    + ": " + data[0]);
-
-                Object[] o =
+                        + ": " + data[0]);
+                
+            	Pair<String, ParserResult> importResult = 
                     Globals.importFormatReader.importUnknownFormat(data[0]
-                        .replaceAll("~", System.getProperty("user.home")));
-                String formatName = (String) o[0];
-
-                if (formatName.equals(ImportFormatReader.BIBTEX_FORMAT)) {
-                    ParserResult pr = (ParserResult)o[1];
-                    pr.setToOpenTab(true);
-                    //loaded.add(pr);
-                    return pr;
-                }
-                else {
-                    List entries = (java.util.List) o[1];
-                    if (entries != null)
-                        System.out.println(Globals.lang("Format used") + ": "
-                            + formatName);
-                    else
-                        System.out.println(Globals.lang(
+                            .replaceAll("~", System.getProperty("user.home")));
+            	
+            	if (importResult != null){
+            		System.out.println(Globals.lang("Format used") + ": "
+                        + importResult.p);
+            		
+            		return importResult.v;
+            	} else {
+            		System.out.println(Globals.lang(
                                 "Could not find a suitable import format."));
-
-                    if (entries != null) {
-                        BibtexDatabase base = ImportFormatReader.createDatabase(entries);
-                        ParserResult pr = new ParserResult(base, null, new HashMap());
-
-                        //pr.setFile(new File(data[0]));
-                        pr.setToOpenTab(true);
-                        //loaded.add(pr);
-                        return pr;
-                    }
                 }
-
             }
         } catch (IOException ex) {
             System.err.println(Globals.lang("Error opening file") + " '"
-                + data[0] + "': " + ex.getMessage());
+                    + data[0] + "': " + ex.getLocalizedMessage());
         }
-
         return null;
     }
+
+    /**
+     * Will open a file (like importFile), but will also request JabRef to focus on this database 
+     * @param argument See importFile.
+     * @return ParserResult with setToOpenTab(true)
+     */
+    public static ParserResult importToOpenBase(String argument) {
+    	ParserResult result = importFile(argument);
+    	
+    	if (result != null)
+    		result.setToOpenTab(true);
+    	
+    	return result;
+    }
 }
