@@ -27,21 +27,38 @@ http://www.gnu.org/copyleft/gpl.ja.html
 
 package net.sf.jabref;
 
-import java.io.*;
-import javax.swing.*;
-
-import net.sf.jabref.labelPattern.DefaultLabelPatterns;
-import net.sf.jabref.labelPattern.LabelPattern;
-import net.sf.jabref.export.CustomExportList;
-import net.sf.jabref.imports.CustomImportList;
-import java.awt.*;
-import java.util.prefs.*;
-import java.util.*;
+import java.awt.Color;
+import java.awt.event.KeyEvent;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.io.Reader;
+import java.io.StringReader;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
-import java.awt.event.*;
+import java.util.TreeSet;
+import java.util.Vector;
+import java.util.prefs.BackingStoreException;
+import java.util.prefs.InvalidPreferencesFormatException;
+import java.util.prefs.Preferences;
+
+import javax.swing.JTable;
+import javax.swing.KeyStroke;
+
+import net.sf.jabref.export.CustomExportList;
 import net.sf.jabref.export.ExportComparator;
 import net.sf.jabref.external.ExternalFileType;
 import net.sf.jabref.external.UnknownExternalFileType;
+import net.sf.jabref.gui.PersistenceTableColumnListener;
+import net.sf.jabref.imports.CustomImportList;
+import net.sf.jabref.labelPattern.DefaultLabelPatterns;
+import net.sf.jabref.labelPattern.LabelPattern;
 
 public class JabRefPreferences {
 
@@ -59,11 +76,12 @@ public class JabRefPreferences {
     public String WRAPPED_USERNAME;
 
     Preferences prefs;
-    public HashMap defaults = new HashMap(),
-        keyBinds = new HashMap(),
-        defKeyBinds = new HashMap();
-    private HashSet putBracesAroundCapitalsFields = new HashSet(4);
-    private HashSet nonWrappableFields = new HashSet(4);
+    public HashMap<String, Object> defaults = new HashMap<String, Object>();
+    public HashMap<String, String>
+        keyBinds = new HashMap<String, String>(),
+        defKeyBinds = new HashMap<String, String>();
+    private HashSet<String> putBracesAroundCapitalsFields = new HashSet<String>(4);
+    private HashSet<String> nonWrappableFields = new HashSet<String>(5);
     private static final LabelPattern KEY_PATTERN = new DefaultLabelPatterns();
     private static LabelPattern keyPattern;
 
@@ -77,7 +95,7 @@ public class JabRefPreferences {
     private EntryEditorTabList tabList = null;
 
     // Map containing all registered external file types:
-    private TreeSet externalFileTypes = new TreeSet();
+    private TreeSet<ExternalFileType> externalFileTypes = new TreeSet<ExternalFileType>();
 
     // The following field is used as a global variable during the export of a database.
     // By setting this field to the path of the database's default file directory, formatters
@@ -86,6 +104,11 @@ public class JabRefPreferences {
     // string to be formatted and possible formatter arguments.
     public String fileDirForDatabase = null;
 
+    // The following field is used as a global variable during the export of a database.
+    // It is used to hold custom name formatters defined by a custom export filter.
+    // It is set before the export starts:
+    public HashMap<String,String> customExportNameFormatters = null;
+
     // The only instance of this class:
     private static JabRefPreferences singleton = null;
 
@@ -98,8 +121,15 @@ public class JabRefPreferences {
     // The constructor is made private to enforce this as a singleton class:
     private JabRefPreferences() {
 
+        try {
+            if (new File("jabref.xml").exists()){
+                importPreferences("jabref.xml");
+            }
+        } catch (IOException e) {
+            Globals.logger("Could not import preferences from jabref.xml:" + e.getLocalizedMessage());
+        }
+        
         prefs = Preferences.userNodeForPackage(JabRef.class);
-        //Util.pr(prefs.toString());
         
         if (Globals.osName.equals(Globals.MAC)) {
 			defaults.put("pdfviewer", "/Applications/Preview.app");
@@ -109,13 +139,13 @@ public class JabRefPreferences {
 			defaults.put("pdfviewer", "cmd.exe /c start /b");
 			defaults.put("psviewer", "cmd.exe /c start /b");
 			defaults.put("htmlviewer", "cmd.exe /c start /b");
-			defaults.put("lookAndFeel", "com.jgoodies.plaf.windows.ExtWindowsLookAndFeel");
+			defaults.put("lookAndFeel", "com.jgoodies.looks.windows.WindowsLookAndFeel");
             defaults.put("winEdtPath", "C:\\Program Files\\WinEdt Team\\WinEdt\\WinEdt.exe");
             defaults.put("latexEditorPath", "C:\\Program Files\\LEd\\LEd.exe");
         } else {
 			defaults.put("pdfviewer", "evince");
 			defaults.put("psviewer", "gv");
-			defaults.put("htmlviewer", "mozilla");
+			defaults.put("htmlviewer", "firefox");
 			defaults.put("lookAndFeel", "com.jgoodies.plaf.plastic.Plastic3DLookAndFeel");
 		}
         defaults.put("useDefaultLookAndFeel", Boolean.TRUE);
@@ -126,8 +156,9 @@ public class JabRefPreferences {
         defaults.put("posY", new Integer(0));
         defaults.put("sizeX", new Integer(840));
         defaults.put("sizeY", new Integer(680));
+        defaults.put("windowMaximised", Boolean.FALSE);
         defaults.put("rememberWindowLocation", Boolean.TRUE);
-        defaults.put("autoResizeMode", new Integer(JTable.AUTO_RESIZE_OFF));
+        defaults.put("autoResizeMode", new Integer(JTable.AUTO_RESIZE_ALL_COLUMNS));
         defaults.put("tableColorCodesOn", Boolean.TRUE);
         defaults.put("namesAsIs", Boolean.FALSE);
         defaults.put("namesFf", Boolean.FALSE);
@@ -146,6 +177,8 @@ public class JabRefPreferences {
         defaults.put("terDescending", Boolean.FALSE);
         defaults.put("columnNames", "entrytype;author;title;year;journal;owner;timestamp;bibtexkey");
         defaults.put("columnWidths","75;280;400;60;100;100;100;100");
+        defaults.put(PersistenceTableColumnListener.ACTIVATE_PREF_KEY, 
+        		new Boolean(PersistenceTableColumnListener.DEFAULT_ENABLED));
         defaults.put("xmpPrivacyFilters", "pdf;timestamp;keywords;owner;note;review");
         defaults.put("useXmpPrivacyFilter", Boolean.FALSE);
         defaults.put("numberColWidth",new Integer(GUIGlobals.NUMBER_COL_LENGTH));
@@ -175,12 +208,14 @@ public class JabRefPreferences {
         defaults.put("incrementS", Boolean.FALSE);
         defaults.put("saveInStandardOrder", Boolean.TRUE);
         defaults.put("saveInOriginalOrder", Boolean.FALSE);
+        defaults.put("exportInStandardOrder", Boolean.TRUE);
+        defaults.put("exportInOriginalOrder", Boolean.FALSE);
         defaults.put("selectS", Boolean.FALSE);
         defaults.put("regExpSearch", Boolean.TRUE);
         defaults.put("searchPanePosX", new Integer(0));
         defaults.put("searchPanePosY", new Integer(0));
         defaults.put("autoComplete", Boolean.TRUE);
-        defaults.put("autoCompleteFields", "author;editor;title;journal;publisher;keywords");
+        defaults.put("autoCompleteFields", "author;editor;title;journal;publisher;keywords;crossref");
         defaults.put("groupSelectorVisible", Boolean.TRUE);
         defaults.put("groupFloatSelections", Boolean.TRUE);
         defaults.put("groupIntersectSelections", Boolean.TRUE);
@@ -201,6 +236,9 @@ public class JabRefPreferences {
         defaults.put("groupsVisibleRows", new Integer(8));
         defaults.put("defaultOwner", System.getProperty("user.name"));
         defaults.put("preserveFieldFormatting", Boolean.FALSE);
+        defaults.put("memoryStickMode", Boolean.FALSE);
+        defaults.put("renameOnMoveFileToFileDir", Boolean.TRUE);
+
     // The general fields stuff is made obsolete by the CUSTOM_TAB_... entries.
         defaults.put("generalFields", "crossref;keywords;file;doi;url;urldate;citeseerurl;"+
                      "pdf;comment;owner");
@@ -260,6 +298,8 @@ public class JabRefPreferences {
         defaults.put("warnBeforeOverwritingKey", Boolean.TRUE);
         defaults.put("confirmDelete", Boolean.TRUE);
         defaults.put("grayOutNonHits", Boolean.TRUE);
+        defaults.put("showSearchInDialog", Boolean.FALSE);
+        defaults.put("searchAllBases", Boolean.FALSE);
         defaults.put("defaultLabelPattern", "[auth][year]");
         defaults.put("previewEnabled", Boolean.TRUE);
         defaults.put("preview0", "<font face=\"arial\">"
@@ -297,14 +337,18 @@ public class JabRefPreferences {
                      +"\\begin{abstract}<BR><BR><b>Abstract: </b> \\format[HTMLChars]{\\abstract} \\end{abstract}__NEWLINE__"
                      +"\\begin{review}<BR><BR><b>Review: </b> \\format[HTMLChars]{\\review} \\end{review}"
                      +"</dd>__NEWLINE__<p></p></font>");
+        // TODO: Currently not possible to edit this setting:
+        defaults.put("previewPrintButton", Boolean.FALSE);
         defaults.put("autoDoubleBraces", Boolean.FALSE);
         defaults.put("doNotResolveStringsFor", "url");
         defaults.put("resolveStringsAllFields", Boolean.FALSE);
         defaults.put("putBracesAroundCapitals","");//"title;journal;booktitle;review;abstract");
-        defaults.put("nonWrappableFields", "pdf;ps;url;doi");
+        defaults.put("nonWrappableFields", "pdf;ps;url;doi;file");
         defaults.put("useImportInspectionDialog", Boolean.TRUE);
-        defaults.put("useImportInspectionDialogForSingle", Boolean.FALSE);
+        defaults.put("useImportInspectionDialogForSingle", Boolean.TRUE);
         defaults.put("generateKeysAfterInspection", Boolean.TRUE);
+        defaults.put("markImportedEntries", Boolean.TRUE);
+        defaults.put("unmarkAllEntriesBeforeImporting", Boolean.TRUE);
         defaults.put("warnAboutDuplicatesInInspection", Boolean.TRUE);
         defaults.put("useTimeStamp", Boolean.TRUE);
         defaults.put("overwriteTimeStamp", Boolean.FALSE);
@@ -318,7 +362,11 @@ public class JabRefPreferences {
 
         defaults.put("personalJournalList", null);
         defaults.put("externalJournalLists", null);
-        defaults.put("citeCommand", "cite");
+        defaults.put("citeCommand", "cite"); // obsoleted by the app-specific ones
+        defaults.put("citeCommandVim", "\\cite");
+        defaults.put("citeCommandEmacs", "\\cite");
+        defaults.put("citeCommandWinEdt", "\\cite");
+        defaults.put("citeCommandLed", "\\cite");
         defaults.put("floatMarkedEntries", Boolean.TRUE);
 
         defaults.put("useNativeFileDialogOnMac", Boolean.FALSE);
@@ -328,10 +376,19 @@ public class JabRefPreferences {
 
         defaults.put("importInspectionDialogWidth", new Integer(650));
         defaults.put("importInspectionDialogHeight", new Integer(650));
-
+        defaults.put("searchDialogWidth", new Integer(650));
+        defaults.put("searchDialogHeight", new Integer(500));
         defaults.put("showFileLinksUpgradeWarning", Boolean.TRUE);
-
         defaults.put("autolinkExactKeyOnly", Boolean.TRUE);
+        defaults.put("numericFields", "mittnum;author");
+        defaults.put("runAutomaticFileSearch", Boolean.FALSE);
+
+        defaults.put("autoSave", Boolean.TRUE);
+        defaults.put("autoSaveInterval", 5);
+        defaults.put("promptBeforeUsingAutosave", Boolean.TRUE);
+
+        defaults.put("deletePlugins", "");
+        
         //defaults.put("lastAutodetectedImport", "");
 
         //defaults.put("autoRemoveExactDuplicates", Boolean.FALSE);
@@ -344,7 +401,7 @@ public class JabRefPreferences {
 
         restoreKeyBindings();
 
-        customExports = new CustomExportList(this, new ExportComparator());
+        customExports = new CustomExportList(new ExportComparator());
         customImports = new CustomImportList(this);
 
         //defaults.put("oooWarning", Boolean.TRUE);
@@ -355,6 +412,7 @@ public class JabRefPreferences {
         defaults.put(DEFAULT_REG_EXP_SEARCH_EXPRESSION_KEY, defaultExpression);
         defaults.put(REG_EXP_SEARCH_EXPRESSION_KEY, defaultExpression);
         defaults.put(USE_REG_EXP_SEARCH_KEY, Boolean.FALSE);
+        defaults.put("useIEEEAbrv", Boolean.TRUE);
     }
     
     public static final String DEFAULT_REG_EXP_SEARCH_EXPRESSION_KEY = "defaultRegExpSearchExpression";
@@ -371,14 +429,14 @@ public class JabRefPreferences {
         if (fieldString.length() > 0) {
             String[] fields = fieldString.split(";");
             for (int i=0; i<fields.length; i++)
-                putBracesAroundCapitalsFields.add(fields[i]);
+                putBracesAroundCapitalsFields.add(fields[i].trim());
         }
         nonWrappableFields.clear();
         fieldString = get("nonWrappableFields");
         if (fieldString.length() > 0) {
             String[] fields = fieldString.split(";");
             for (int i=0; i<fields.length; i++)
-                nonWrappableFields.add(fields[i]);
+                nonWrappableFields.add(fields[i].trim());
         }
 
     }
@@ -401,21 +459,37 @@ public class JabRefPreferences {
     }
 
     public boolean getBoolean(String key) {
-        return prefs.getBoolean(key, ((Boolean)defaults.get(key)).booleanValue());
+        return prefs.getBoolean(key, getBooleanDefault(key));
+    }
+    
+    public boolean getBooleanDefault(String key){
+        return ((Boolean)defaults.get(key)).booleanValue();
     }
 
     public double getDouble(String key) {
-        return prefs.getDouble(key, ((Double)defaults.get(key)).doubleValue());
+        return prefs.getDouble(key, getDoubleDefault(key));
+    }
+    
+    public double getDoubleDefault(String key){
+        return ((Double)defaults.get(key)).doubleValue();
     }
 
     public int getInt(String key) {
-        return prefs.getInt(key, ((Integer)defaults.get(key)).intValue());
+        return prefs.getInt(key, getIntDefault(key));
     }
 
+    public int getIntDefault(String key) {
+        return ((Integer)defaults.get(key)).intValue();
+    }
+    
     public byte[] getByteArray(String key) {
-        return prefs.getByteArray(key, (byte[])defaults.get(key));
+        return prefs.getByteArray(key, getByteArrayDefault(key));
     }
 
+    public byte[] getByteArrayDefault(String key){
+        return (byte[])defaults.get(key);   
+    }
+    
     public void put(String key, String value) {
         prefs.put(key, value);
     }
@@ -473,7 +547,7 @@ public class JabRefPreferences {
             return null;
 
         StringReader rd = new StringReader(names);
-        Vector arr = new Vector();
+        Vector<String> arr = new Vector<String>();
         String rs;
         try {
             while ((rs = getNextUnit(rd)) != null) {
@@ -482,7 +556,7 @@ public class JabRefPreferences {
         } catch (IOException ex) {}
         String[] res = new String[arr.size()];
         for (int i=0; i<res.length; i++)
-            res[i] = (String)arr.elementAt(i);
+            res[i] = arr.elementAt(i);
 
         return res;
     }
@@ -504,6 +578,16 @@ public class JabRefPreferences {
         return new Color(rgb[0], rgb[1], rgb[2]);
     }
 
+    /**
+     * Set the default value for a key. This is useful for plugins that need to
+     * add default values for the prefs keys they use.
+     * @param key The preferences key.
+     * @param value The default value.
+     */
+    public void putDefaultValue(String key, Object value) {
+        defaults.put(key, value);
+    }
+
     /**
      * Stores a color in preferences.
      * @param key The key for this setting.
@@ -539,13 +623,13 @@ public class JabRefPreferences {
      */
     public KeyStroke getKey(String bindName) {
 
-        String s = (String)keyBinds.get(bindName);
+        String s = keyBinds.get(bindName);
         // If the current key bindings don't contain the one asked for,
         // we fall back on the default. This should only happen when a
         // user has his own set in Preferences, and has upgraded to a
         // new version where new bindings have been introduced.
         if (s == null) {
-            s = (String)defKeyBinds.get(bindName);
+            s = defKeyBinds.get(bindName);
             // So, if this happens, we add the default value to the current
             // hashmap, so this doesn't happen again, and so this binding
             // will appear in the KeyBindingsDialog.
@@ -553,7 +637,6 @@ public class JabRefPreferences {
         }
         if (s == null) {
           Globals.logger("Could not get key binding for \"" + bindName + "\"");
-          //throw new RuntimeException("");
         }
 
         if (Globals.ON_MAC)
@@ -575,48 +658,58 @@ public class JabRefPreferences {
       }
       else {
         if ((ks.getModifiers() & KeyEvent.SHIFT_MASK) != 0) {
-          return KeyStroke.getKeyStroke(keyCode, Globals.SHORTCUT_MASK+KeyEvent.SHIFT_MASK);
+          return KeyStroke.getKeyStroke(keyCode, Globals.getShortcutMask()+KeyEvent.SHIFT_MASK);
         }
-        return KeyStroke.getKeyStroke(keyCode, Globals.SHORTCUT_MASK);
+        return KeyStroke.getKeyStroke(keyCode, Globals.getShortcutMask());
       }
     }
 
     /**
      * Returns the HashMap containing all key bindings.
      */
-    public HashMap getKeyBindings() {
+    public HashMap<String, String> getKeyBindings() {
         return keyBinds;
     }
 
     /**
      * Returns the HashMap containing default key bindings.
      */
-    public HashMap getDefaultKeys() {
+    public HashMap<String, String> getDefaultKeys() {
         return defKeyBinds;
     }
 
+    /**
+     * Calling this method will write all preferences into the preference store.
+     */
     public void flush() {
-    try {
-        prefs.flush();
-    } catch (BackingStoreException ex) {
-        ex.printStackTrace();
-    }
+        if (getBoolean("memoryStickMode")){
+            try {
+                exportPreferences("jabref.xml");
+            } catch (IOException e) {
+                Globals.logger("Could not save preferences for memory stick mode: " + e.getLocalizedMessage());
+            }
+        }
+        try {
+            prefs.flush();
+        } catch (BackingStoreException ex) {
+            ex.printStackTrace();
+        }
     }
 
     /**
      * Stores new key bindings into Preferences, provided they
      * actually differ from the old ones.
      */
-    public void setNewKeyBindings(HashMap newBindings) {
+    public void setNewKeyBindings(HashMap<String, String> newBindings) {
         if (!newBindings.equals(keyBinds)) {
             // This confirms that the bindings have actually changed.
             String[] bindNames = new String[newBindings.size()],
                 bindings = new String[newBindings.size()];
             int index = 0;
-            for (Iterator i=newBindings.keySet().iterator();
+            for (Iterator<String> i=newBindings.keySet().iterator();
                  i.hasNext();) {
-                String nm = (String)i.next();
-                String bnd = (String)newBindings.get(nm);
+                String nm = i.next();
+                String bnd = newBindings.get(nm);
                 bindNames[index] = nm;
                 bindings[index] = bnd;
                 index++;
@@ -664,9 +757,7 @@ public class JabRefPreferences {
                 Globals.logger("BackingStoreException in JabRefPreferences.putKeyPattern");
             }
 
-            Iterator i = pattern.keySet().iterator();
-            while (i.hasNext()) {
-                String s = (String)i.next();
+            for (String s: pattern.keySet()){
                 if (!(pattern.get(s)).equals(parent.get(s)))
                     pre.put(s, pattern.getValue(s).get(0).toString());
             }
@@ -700,7 +791,6 @@ public class JabRefPreferences {
         defKeyBinds.put("Push to application","ctrl L");
       defKeyBinds.put("Push to LyX","ctrl L");
       defKeyBinds.put("Push to WinEdt","ctrl shift W");
-      defKeyBinds.put("Push to Emacs","ctrl shift E");
         defKeyBinds.put("Quit JabRef", "ctrl Q");
         defKeyBinds.put("Open database", "ctrl O");
         defKeyBinds.put("Save database", "ctrl S");
@@ -750,6 +840,7 @@ public class JabRefPreferences {
         defKeyBinds.put("Save session", "F11");
         defKeyBinds.put("Load session", "F12");
         defKeyBinds.put("Copy \\cite{BibTeX key}", "ctrl K");
+        defKeyBinds.put("Copy BibTeX key", "ctrl shift K");
         defKeyBinds.put("Next tab", "ctrl PAGE_DOWN");
         defKeyBinds.put("Previous tab", "ctrl PAGE_UP");
         defKeyBinds.put("Replace string", "ctrl R");
@@ -757,6 +848,7 @@ public class JabRefPreferences {
         defKeyBinds.put("Open file", "F4");
         defKeyBinds.put("Open PDF or PS", "shift F5");
         defKeyBinds.put("Open URL or DOI", "F3");
+        defKeyBinds.put("Open SPIRES entry", "ctrl F3");
         defKeyBinds.put("Toggle entry preview", "ctrl F9");
         defKeyBinds.put("Switch preview layout", "F9");
         defKeyBinds.put("Edit entry", "ctrl E");
@@ -770,14 +862,26 @@ public class JabRefPreferences {
         defKeyBinds.put("Synchronize files", "ctrl F4");
         defKeyBinds.put("Synchronize PDF", "shift F4");
         defKeyBinds.put("Synchronize PS", "ctrl shift F4");
+        defKeyBinds.put("Focus entry table", "ctrl shift E");
 
         defKeyBinds.put("Abbreviate", "ctrl alt A");
         defKeyBinds.put("Unabbreviate", "ctrl alt shift A");
         defKeyBinds.put("Search IEEEXplore", "F8");
+        defKeyBinds.put("Search ACM Portal", "ctrl shift F8");
         defKeyBinds.put("Fetch ArXiv.org", "shift F8");
+        defKeyBinds.put("Search JSTOR", "shift F9");
+        defKeyBinds.put("Fetch SPIRES", "ctrl F8");
         defKeyBinds.put("Write XMP", "ctrl F4");
         defKeyBinds.put("New file link", "ctrl N");
-        //defKeyBinds.put("Select value", "ctrl B");
+        defKeyBinds.put("Fetch SPIRES", "ctrl F8");
+        defKeyBinds.put("Back", "alt LEFT");
+        defKeyBinds.put("Forward", "alt RIGHT");
+        defKeyBinds.put("Import into current database", "ctrl I");
+        defKeyBinds.put("Import into new database", "ctrl shift I");
+
+        defKeyBinds.put("Increase table font size", "ctrl PLUS");
+        defKeyBinds.put("Decrease table font size", "ctrl MINUS");
+
     }
 
     private String getNextUnit(Reader data) throws IOException {
@@ -857,40 +961,39 @@ public class JabRefPreferences {
 
     public List<ExternalFileType> getDefaultExternalFileTypes() {
         List<ExternalFileType> list = new ArrayList<ExternalFileType>();
-        list.add(new ExternalFileType("PDF", "pdf", "evince", "pdfSmall"));
-        list.add(new ExternalFileType("PostScript", "ps", "evince", "psSmall"));
-        list.add(new ExternalFileType("Word", "doc", "oowriter", "openoffice"));
-        list.add(new ExternalFileType("OpenDocument text", "odt", "oowriter", "openoffice"));
-        list.add(new ExternalFileType("Excel", "xls", "oocalc", "openoffice"));
-        list.add(new ExternalFileType("OpenDocument spreadsheet", "ods", "oocalc", "openoffice"));
-        list.add(new ExternalFileType("PowerPoint", "ppt", "ooimpress", "openoffice"));
-        list.add(new ExternalFileType("OpenDocument presentation", "odp", "ooimpress", "openoffice"));
-        list.add(new ExternalFileType("Rich Text Format", "rtf", "oowriter", "openoffice"));
-        list.add(new ExternalFileType("PNG image", "png", "gimp", "picture"));
-        list.add(new ExternalFileType("GIF image", "gif", "gimp", "picture"));
-        list.add(new ExternalFileType("JPG image", "jpg", "gimp", "picture"));
-        list.add(new ExternalFileType("Djvu", "djvu", "evince", "psSmall"));
-        list.add(new ExternalFileType("Text", "txt", "emacs", "emacs"));
-        list.add(new ExternalFileType("LaTeX", "tex", "emacs", "emacs"));
-        list.add(new ExternalFileType("CHM", "chm", "gnochm", "www"));
-        list.add(new ExternalFileType("TIFF image", "tiff", "gimp", "picture"));
-        ExternalFileType tp = new ExternalFileType("URL", "html", "firefox", "www");
+        list.add(new ExternalFileType("PDF", "pdf", "application/pdf", "evince", "pdfSmall"));
+        list.add(new ExternalFileType("PostScript", "ps", "application/postscript", "evince", "psSmall"));
+        list.add(new ExternalFileType("Word", "doc", "application/msword", "oowriter", "openoffice"));
+        list.add(new ExternalFileType("OpenDocument text", "odt", "application/vnd.oasis.opendocument.text", "oowriter", "openoffice"));
+        list.add(new ExternalFileType("Excel", "xls", "application/excel", "oocalc", "openoffice"));
+        list.add(new ExternalFileType("OpenDocument spreadsheet", "ods", "application/vnd.oasis.opendocument.spreadsheet", "oocalc", "openoffice"));
+        list.add(new ExternalFileType("PowerPoint", "ppt", "", "ooimpress", "openoffice"));
+        list.add(new ExternalFileType("OpenDocument presentation", "odp", "application/vnd.oasis.opendocument.presentation", "ooimpress", "openoffice"));
+        list.add(new ExternalFileType("Rich Text Format", "rtf", "application/rtf", "oowriter", "openoffice"));
+        list.add(new ExternalFileType("PNG image", "png", "image/png", "gimp", "picture"));
+        list.add(new ExternalFileType("GIF image", "gif", "image/gif", "gimp", "picture"));
+        list.add(new ExternalFileType("JPG image", "jpg", "image/jpeg", "gimp", "picture"));
+        list.add(new ExternalFileType("Djvu", "djvu", "", "evince", "psSmall"));
+        list.add(new ExternalFileType("Text", "txt", "text/plain", "emacs", "emacs"));
+        list.add(new ExternalFileType("LaTeX", "tex", "", "emacs", "emacs"));
+        list.add(new ExternalFileType("CHM", "chm", "", "gnochm", "www"));
+        list.add(new ExternalFileType("TIFF image", "tiff", "image/tiff", "gimp", "picture"));
+        ExternalFileType tp = new ExternalFileType("URL", "html", "text/html", "firefox", "www");
         list.add(tp);
 
-        // Under Windows/Mac we initialize all file types with an empty viewer app, to
-        // rely on the default app instead:
-        if (Globals.ON_WIN || Globals.ON_MAC) {
-            for (Iterator<ExternalFileType> iterator = list.iterator(); iterator.hasNext();) {
-                ExternalFileType type = iterator.next();
-                type.setOpenWith("");
-            }
+        // On all OSes there is a generic application available to handle file opening,
+        // so we don't need the default application settings anymore:
+        for (Iterator<ExternalFileType> iterator = list.iterator(); iterator.hasNext();) {
+            ExternalFileType type = iterator.next();
+            type.setOpenWith("");
         }
+        
 
         return list;
     }
 
     public ExternalFileType[] getExternalFileTypeSelection() {
-        return (ExternalFileType[])externalFileTypes.toArray
+        return externalFileTypes.toArray
                 (new ExternalFileType[externalFileTypes.size()]);
     }
 
@@ -900,8 +1003,8 @@ public class JabRefPreferences {
      * @return The ExternalFileType registered, or null if none.
      */
     public ExternalFileType getExternalFileTypeByName(String name) {
-        for (Iterator iterator = externalFileTypes.iterator(); iterator.hasNext();) {
-            ExternalFileType type = (ExternalFileType) iterator.next();
+        for (Iterator<ExternalFileType> iterator = externalFileTypes.iterator(); iterator.hasNext();) {
+            ExternalFileType type = iterator.next();
             if (type.getName().equals(name))
                 return type;
         }
@@ -915,14 +1018,27 @@ public class JabRefPreferences {
      * @return The ExternalFileType registered, or null if none.
      */
     public ExternalFileType getExternalFileTypeByExt(String extension) {
-        for (Iterator iterator = externalFileTypes.iterator(); iterator.hasNext();) {
-            ExternalFileType type = (ExternalFileType) iterator.next();
-            if (type.getExtension().equals(extension))
+        for (Iterator<ExternalFileType> iterator = externalFileTypes.iterator(); iterator.hasNext();) {
+            ExternalFileType type = iterator.next();
+            if ((type.getExtension() != null) && type.getExtension().equalsIgnoreCase(extension))
                 return type;
         }
         return null;
     }
 
+    /**
+     * Look up the external file type registered for this MIME type, if any.
+     * @param mimeType The MIME type.
+     * @return The ExternalFileType registered, or null if none.
+     */
+    public ExternalFileType getExternalFileTypeByMimeType(String mimeType) {
+        for (Iterator<ExternalFileType> iterator = externalFileTypes.iterator(); iterator.hasNext();) {
+            ExternalFileType type = iterator.next();
+            if ((type.getMimeType() != null) && type.getMimeType().equals(mimeType))
+                return type;
+        }
+        return null;
+    }
 
     /**
      * Reset the List of external file types after user customization.
