@@ -3,17 +3,9 @@ package net.sf.jabref;
 import java.awt.BorderLayout;
 import java.awt.Dimension;
 import java.awt.event.ActionEvent;
-import java.util.Vector;
+import java.util.*;
 
-import javax.swing.AbstractAction;
-import javax.swing.BorderFactory;
-import javax.swing.JButton;
-import javax.swing.JLabel;
-import javax.swing.JPanel;
-import javax.swing.JScrollPane;
-import javax.swing.JTable;
-import javax.swing.JToolBar;
-import javax.swing.SwingConstants;
+import javax.swing.*;
 import javax.swing.table.AbstractTableModel;
 import javax.swing.table.TableColumnModel;
 import javax.swing.table.TableModel;
@@ -27,7 +19,7 @@ class TableColumnsTab extends JPanel implements PrefsTab {
     private boolean tableChanged = false;
     private JTable colSetup;
     private int rowCount = -1, ncWidth = -1;
-    private Vector tableRows = new Vector(10);
+    private Vector<TableRow> tableRows = new Vector<TableRow>(10);
     private JabRefFrame frame;
 
     class TableRow {
@@ -83,9 +75,11 @@ class TableColumnsTab extends JPanel implements PrefsTab {
                 public String getColumnName(int col) {
                     return (col == 0 ? Globals.lang("Field name") : Globals.lang("Column width"));
                 }
-                public Class getColumnClass(int column) {
-                    if (column == 0) return String.class;
-                    else return Integer.class;
+                public Class<?> getColumnClass(int column) {
+                    if (column == 0)
+                    	return String.class;
+                    else
+                    	return Integer.class;
                 }
                 public boolean isCellEditable(int row, int col) {
                     return !((row == 0) && (col == 0));
@@ -101,7 +95,7 @@ class TableColumnsTab extends JPanel implements PrefsTab {
                           return;
                         }
 
-                    TableRow rowContent = (TableRow)tableRows.elementAt(row-1);
+                    TableRow rowContent = tableRows.elementAt(row-1);
 
                     if (col == 0) {
                         rowContent.name = value.toString();
@@ -140,9 +134,14 @@ class TableColumnsTab extends JPanel implements PrefsTab {
         //tlb.setLayout(gbl);
         AddRowAction ara = new AddRowAction();
         DeleteRowAction dra = new DeleteRowAction();
+        MoveRowUpAction moveUp = new MoveRowUpAction();
+        MoveRowDownAction moveDown = new MoveRowDownAction();
         tlb.setBorder(null);
         tlb.add(ara);
         tlb.add(dra);
+        tlb.addSeparator();
+        tlb.add(moveUp);
+        tlb.add(moveDown);
         //tlb.addSeparator();
         //tlb.add(new UpdateWidthsAction());
         tabPanel.add(tlb, BorderLayout.EAST);
@@ -154,9 +153,12 @@ class TableColumnsTab extends JPanel implements PrefsTab {
 //						+"to match the current widths in your table")+")</HTML>");
 //        lab = new JLabel("<HTML>("+Globals.lang("this_button_will_update") +")</HTML>") ;
         builder.append(pan);
-        JButton button = new JButton(new UpdateWidthsAction());
-        builder.append(button); builder.nextLine();
-        builder.append(pan); 
+        JButton buttonWidth = new JButton(new UpdateWidthsAction());
+        JButton buttonOrder = new JButton(new UpdateOrderAction());
+        builder.append(buttonWidth);builder.nextLine();
+        builder.append(pan);
+        builder.append(buttonOrder);builder.nextLine();
+        builder.append(pan);
         //builder.append(lab);
         builder.nextLine();
         pan = builder.getPanel();
@@ -230,6 +232,120 @@ class TableColumnsTab extends JPanel implements PrefsTab {
         }
     }
 
+    abstract class AbstractMoveRowAction extends AbstractAction {
+		public AbstractMoveRowAction(String string, ImageIcon image) {
+			super(string, image);
+		}
+
+		protected void swap(int i, int j) {
+			if (i < 0 || i >= tableRows.size())
+				return;
+			if (j < 0 || j >= tableRows.size())
+				return;
+			TableRow tmp = tableRows.get(i);
+			tableRows.set(i, tableRows.get(j));
+			tableRows.set(j, tmp);
+		}
+	}
+
+	class MoveRowUpAction extends AbstractMoveRowAction {
+		public MoveRowUpAction() {
+			super("Up", GUIGlobals.getImage("up"));
+			putValue(SHORT_DESCRIPTION, Globals.lang("Move up"));
+		}
+
+		public void actionPerformed(ActionEvent e) {
+			int selected[] = colSetup.getSelectedRows();
+			Arrays.sort(selected);
+			// first element (#) not inside tableRows
+			// don't move if a selected element is at bounce
+			if (selected.length > 0 && selected[0] > 1) {
+				boolean newSelected[] = new boolean[colSetup.getRowCount()];
+				for (int i : selected) {
+					swap(i - 1, i - 2);
+					newSelected[i - 1] = true;
+				}
+				// select all and remove unselected
+				colSetup.setRowSelectionInterval(0, colSetup.getRowCount() - 1);
+				for (int i = 0; i < colSetup.getRowCount(); i++) {
+					if (!newSelected[i])
+						colSetup.removeRowSelectionInterval(i, i);
+				}
+				colSetup.revalidate();
+				colSetup.repaint();
+				tableChanged = true;
+			}
+		}
+	}
+
+	class MoveRowDownAction extends AbstractMoveRowAction {
+		public MoveRowDownAction() {
+			super("Down", GUIGlobals.getImage("down"));
+			putValue(SHORT_DESCRIPTION, Globals.lang("Down up"));
+		}
+
+		public void actionPerformed(ActionEvent e) {
+			int selected[] = colSetup.getSelectedRows();
+			Arrays.sort(selected);
+			final int last = selected.length - 1;
+			boolean newSelected[] = new boolean[colSetup.getRowCount()];
+			// don't move if a selected element is at bounce
+			if (selected.length > 0 && selected[last] < tableRows.size()) {
+				for (int i = last; i >= 0; i--) {
+					swap(selected[i] - 1, selected[i]);
+					newSelected[selected[i] + 1] = true;
+				}
+				// select all and remove unselected
+				colSetup.setRowSelectionInterval(0, colSetup.getRowCount() - 1);
+				for (int i = 0; i < colSetup.getRowCount(); i++) {
+					if (!newSelected[i])
+						colSetup.removeRowSelectionInterval(i, i);
+				}
+				colSetup.revalidate();
+				colSetup.repaint();
+				tableChanged = true;
+			}
+		}
+	}
+
+	class UpdateOrderAction extends AbstractAction {
+		public UpdateOrderAction() {
+			super(Globals.lang("Update to current column order"));
+		}
+
+		public void actionPerformed(ActionEvent e) {
+			BasePanel panel = frame.basePanel();
+			if (panel == null) {
+				return;
+			}
+			// idea: sort elements according to value stored in hash, keep
+			// everything not inside hash/mainTable as it was
+			final HashMap<String, Integer> map = new HashMap<String, Integer>();
+
+			// first element (#) not inside tableRows
+			for (int i = 1; i < panel.mainTable.getColumnCount(); i++) {
+				String name = panel.mainTable.getColumnName(i);
+				if (name != null && name.length() != 0) {
+					map.put(name.toLowerCase(), i);
+				}
+			}
+			Collections.sort(tableRows, new Comparator<TableRow>() {
+				public int compare(TableRow o1, TableRow o2) {
+					Integer n1 = map.get(o1.name);
+					Integer n2 = map.get(o2.name);
+					if (n1 == null || n2 == null) {
+						return 0;
+					}
+					return n1.compareTo(n2);
+				}
+			});
+
+			colSetup.revalidate();
+			colSetup.repaint();
+			tableChanged = true;
+		}
+	}
+
     class UpdateWidthsAction extends AbstractAction {
         public UpdateWidthsAction() {
           //super(Globals.lang("Update to current column widths"));
@@ -268,7 +384,7 @@ class TableColumnsTab extends JPanel implements PrefsTab {
         }
     }
 
- 
+
     /**
      * Store changes to table preferences. This method is called when
      * the user clicks Ok.
@@ -296,7 +412,7 @@ class TableColumnsTab extends JPanel implements PrefsTab {
             // First we remove all rows with empty names.
             int i=0;
             while (i < tableRows.size()) {
-                if (((TableRow)tableRows.elementAt(i)).name.equals(""))
+                if (tableRows.elementAt(i).name.equals(""))
                     tableRows.removeElementAt(i);
                 else i++;
             }
@@ -307,7 +423,7 @@ class TableColumnsTab extends JPanel implements PrefsTab {
 
             _prefs.putInt("numberColWidth", ncWidth);
             for (i=0; i<tableRows.size(); i++) {
-                TableRow tr = (TableRow)tableRows.elementAt(i);
+                TableRow tr = tableRows.elementAt(i);
                 names[i] = tr.name;
                 nWidths[i] = tr.length;
                 widths[i] = ""+tr.length;
@@ -328,4 +444,4 @@ class TableColumnsTab extends JPanel implements PrefsTab {
 	public String getTabName() {
 	    return Globals.lang("Entry table columns");
 	}
-}
\ No newline at end of file
+}
