@@ -35,20 +35,32 @@ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 package net.sf.jabref.about ;
 
-import java.io.* ;
-import java.util.* ;
+import java.awt.Color;
+import java.awt.Dimension;
+import java.awt.Font;
+import java.awt.FontMetrics;
+import java.awt.Graphics;
+import java.awt.image.FilteredImageSource;
+import java.awt.image.ImageProducer;
+import java.io.BufferedReader;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.util.Enumeration;
+import java.util.Vector;
 
-import java.awt.* ;
-import java.awt.image.* ;
-import javax.swing.* ;
-import javax.swing.border.* ;
+import javax.swing.BorderFactory;
+import javax.swing.ImageIcon;
+import javax.swing.JComponent;
+import javax.swing.UIManager;
+import javax.swing.border.BevelBorder;
 
-import net.sf.jabref.* ;
+import net.sf.jabref.GUIGlobals;
+import net.sf.jabref.Globals;
 
 // AboutPanel class
 public class AboutPanel extends JComponent
 {
-  private Vector textBlocks ;
+  private Vector<TextBlock> textBlocks ;
   private String versionStr ;
   private String buildStr ;
   private AnimationThread thread;
@@ -113,7 +125,7 @@ public class AboutPanel extends JComponent
 
     setBorder ( BorderFactory.createBevelBorder( BevelBorder.RAISED)) ;
 
-    textBlocks = new Vector( 50 ) ;
+    textBlocks = new Vector<TextBlock>( 50 ) ;
 
     loadAboutText() ;
 
@@ -170,12 +182,8 @@ public class AboutPanel extends JComponent
                                                  "credits.txt" ) ;
       }
       InputStreamReader reader = new InputStreamReader( stream ) ;
-//          getClass().getResourceAsStream(GUIGlobals.getLocaleHelpPath() + "credits.txt" ) ) ;
-//          getClass().getResourceAsStream( "/help/credits.txt" ) ) ;
       BufferedReader input = new BufferedReader(reader, 1000) ;
 
-//      System.out.println("encoding :" +reader.getEncoding() ) ;
-
       while ( input.ready() )
       {
         String line = input.readLine() ;
@@ -214,9 +222,10 @@ public class AboutPanel extends JComponent
               aLine.setDirection( 0.0, -1.0 ) ;
               aLine.setFont( font3 ) ;
 
-//              System.out.println( "line " +aLine.getText() +"<" +(borders[3] + ( index * fm.getHeight() * 1.5 )) +">" ) ;
-
+              block = new TextBlock() ;
               block.add( aLine ) ;
+              block.setVisible(true);
+              
               index++ ;
             }
           }
@@ -272,19 +281,14 @@ public class AboutPanel extends JComponent
       g.drawString( "JabRef", (getWidth() - fm.stringWidth("JabRef")) /2, fm.getHeight()+10) ;
 
 
-      for ( Enumeration myE = textBlocks.elements() ; myE.hasMoreElements() ; )
-      {
-        TextBlock block = (TextBlock) myE.nextElement() ;
-
+      for ( TextBlock block : textBlocks){
         if (block.isVisible()) // only if Block is marked as visible
         {
           // print Heading
           AboutTextLine head = block.getHeading() ;
           drawLine(head, g) ;
 
-          for ( Enumeration myEnum = block.getEnumeration() ; myEnum.hasMoreElements() ; )
-          {
-            AboutTextLine line = ( AboutTextLine ) myEnum.nextElement() ;
+          for (AboutTextLine line : block){
             drawLine(line, g) ;
           }
         }
@@ -389,16 +393,14 @@ public class AboutPanel extends JComponent
         {
           int counter = 0 ;
 
-          for ( Enumeration myE = textBlocks.elements() ; myE.hasMoreElements() ; )
+          for ( Enumeration<TextBlock> myE = textBlocks.elements() ; myE.hasMoreElements() ; )
           {
-            TextBlock block = ( TextBlock ) myE.nextElement() ;
+            TextBlock block = myE.nextElement() ;
             AboutTextLine head = block.getHeading() ;
             counter = performStep(head) ;
 
-            for ( Enumeration myEnum = block.getEnumeration() ; myEnum.hasMoreElements() ; )
-            {
-              AboutTextLine line = ( AboutTextLine ) myEnum.nextElement() ;
-              counter += performStep( line ) ;
+            for (AboutTextLine line : block){
+            	counter += performStep( line ) ;
             }
           }
           if (counter < 1)
