@@ -1,12 +1,18 @@
 package net.sf.jabref.collab;
 
-import java.io.*;
-import java.util.*;
-import javax.swing.*;
+import java.io.File;
+import java.io.IOException;
+import java.util.HashSet;
+import java.util.Iterator;
+
+import javax.swing.JOptionPane;
+import javax.swing.SwingUtilities;
 import javax.swing.tree.DefaultMutableTreeNode;
+
 import net.sf.jabref.*;
-import net.sf.jabref.groups.*;
-import net.sf.jabref.imports.*;
+import net.sf.jabref.groups.GroupTreeNode;
+import net.sf.jabref.imports.OpenDatabaseAction;
+import net.sf.jabref.imports.ParserResult;
 
 
 public class ChangeScanner extends Thread {
@@ -93,19 +99,21 @@ public class ChangeScanner extends Thread {
         return changes.getChildCount() > 0;
     }
 
-    public void displayResult() {
+    public void displayResult(final DisplayResultCallback fup) {
         if (changes.getChildCount() > 0) {
             SwingUtilities.invokeLater(new Runnable() {
                 public void run() {
                     ChangeDisplayDialog dial = new ChangeDisplayDialog(frame, panel, changes);
                     Util.placeDialog(dial, frame);
                     dial.setVisible(true); // dial.show(); -> deprecated since 1.5
+                    fup.scanResultsResolved(dial.isOkPressed());
                 }
             });
 
         } else {
             JOptionPane.showMessageDialog(frame, Globals.lang("No actual changes found."),
             Globals.lang("External changes"), JOptionPane.INFORMATION_MESSAGE);
+            fup.scanResultsResolved(true);
         }
     }
 
@@ -118,8 +126,8 @@ public class ChangeScanner extends Thread {
 
         // Create a HashSet where we can put references to entry numbers in the "disk"
         // database that we have matched. This is to avoid matching them twice.
-        HashSet used = new HashSet(disk.getEntryCount());
-        HashSet notMatched = new HashSet(tmp.getEntryCount());
+        HashSet<String> used = new HashSet<String>(disk.getEntryCount());
+        HashSet<Integer> notMatched = new HashSet<Integer>(tmp.getEntryCount());
 
         // Loop through the entries of the "mem" database, looking for exact matches in the "disk" one.
         // We must finish scanning for exact matches before looking for near matches, to avoid an exact
@@ -131,7 +139,7 @@ public class ChangeScanner extends Thread {
             // (if there are not any entries left in the "disk" database, comp will stay at -1,
             // and this entry will be marked as nonmatched).
             if (!used.contains(""+piv2) && (piv2<disk.getEntryCount())) {
-                comp = Util.compareEntriesStrictly(tmp.getEntryAt(piv1), disk.getEntryAt(piv2));
+                comp = DuplicateCheck.compareEntriesStrictly(tmp.getEntryAt(piv1), disk.getEntryAt(piv2));
             }
             if (comp > 1) {
                 used.add(""+piv2);
@@ -143,7 +151,7 @@ public class ChangeScanner extends Thread {
             if (piv2 < disk.getEntryCount()-1) {
                 for (int i = piv2+1; i < disk.getEntryCount(); i++) {
                     if (!used.contains(""+i))
-                        comp = Util.compareEntriesStrictly(tmp.getEntryAt(piv1), disk.getEntryAt(i));
+                        comp = DuplicateCheck.compareEntriesStrictly(tmp.getEntryAt(piv1), disk.getEntryAt(i));
                     else
                         comp = -1;
 
@@ -163,9 +171,9 @@ public class ChangeScanner extends Thread {
         // for close matches.
         if (notMatched.size() > 0) {
 
-            fuzzyLoop: for (Iterator it=notMatched.iterator(); it.hasNext();) {
+            for (Iterator<Integer> it=notMatched.iterator(); it.hasNext();) {
 
-                Integer integ = (Integer)it.next();
+                Integer integ = it.next();
                 piv1 = integ.intValue();
 
 
@@ -178,7 +186,7 @@ public class ChangeScanner extends Thread {
                 if (piv2 < disk.getEntryCount()-1) {
                     for (int i = piv2; i < disk.getEntryCount(); i++) {
                         if (!used.contains(""+i)) {
-                            comp = Util.compareEntriesStrictly(tmp.getEntryAt(piv1),
+                            comp = DuplicateCheck.compareEntriesStrictly(tmp.getEntryAt(piv1),
                             disk.getEntryAt(i));
                         }
                         else
@@ -233,7 +241,7 @@ public class ChangeScanner extends Thread {
                     // See if there is an identical dupe in the mem database:
                     boolean hasAlready = false;
                     for (int j = 0; j < mem.getEntryCount(); j++) {
-                        if (Util.compareEntriesStrictly(mem.getEntryAt(j),
+                        if (DuplicateCheck.compareEntriesStrictly(mem.getEntryAt(j),
                             disk.getEntryAt(i)) >= 1) {
                             hasAlready = true;
                             break;
@@ -265,7 +273,7 @@ public class ChangeScanner extends Thread {
         double comp = -1;
         int found = 0;
         loop: for (int i=0; i<neu.getEntryCount(); i++) {
-            double res = Util.compareEntriesStrictly(old.getEntryAt(index),
+            double res = DuplicateCheck.compareEntriesStrictly(old.getEntryAt(index),
             neu.getEntryAt(i));
             if (res > comp) {
                 comp = res;
@@ -296,19 +304,17 @@ public class ChangeScanner extends Thread {
         if ((nTmp == 0) && (nDisk == 0))
             return;
 
-        HashSet used = new HashSet();
-        HashSet usedInMem = new HashSet();
-        HashSet notMatched = new HashSet(onTmp.getStringCount());
+        HashSet<Object> used = new HashSet<Object>();
+        HashSet<Object> usedInMem = new HashSet<Object>();
+        HashSet<String> notMatched = new HashSet<String>(onTmp.getStringCount());
 
         // First try to match by string names.
         //int piv2 = -1;
-        mainLoop: for (Iterator i=onTmp.getStringKeySet().iterator(); i.hasNext();) {
-            Object tmpId = i.next();
-            BibtexString tmp = onTmp.getString(tmpId);
+        mainLoop: for (String key : onTmp.getStringKeySet()){
+            BibtexString tmp = onTmp.getString(key);
 
             //      for (int j=piv2+1; j<nDisk; j++)
-            for (Iterator j=onDisk.getStringKeySet().iterator(); j.hasNext();) {
-                Object diskId = j.next();
+            for (String diskId : onDisk.getStringKeySet()){
                 if (!used.contains(diskId)) {
                     BibtexString disk = onDisk.getString(diskId);
                     if (disk.getName().equals(tmp.getName())) {
@@ -337,18 +343,14 @@ public class ChangeScanner extends Thread {
 
         // See if we can detect a name change for those entries that we couldn't match.
         if (notMatched.size() > 0) {
-            for (Iterator i = notMatched.iterator(); i.hasNext(); ) {
-                Object nmId = i.next();
-                BibtexString tmp = onTmp.getString(nmId);
+            for (Iterator<String> i = notMatched.iterator(); i.hasNext();){
+                BibtexString tmp = onTmp.getString(i.next());
 
                 // If we get to this point, we found no string with matching name. See if we
                 // can find one with matching content.
-                String tmpContent = tmp.getContent();
-                //for (Iterator i=onTmp.getStringKeySet().iterator(); i.hasNext();) {
-                for (Iterator j=onDisk.getStringKeySet().iterator(); j.hasNext();) {
-                    Object diskId = j.next();
-                    //for (int j = piv2 + 1; j < nDisk; j++)
-                    if (!used.contains(diskId)) {
+                for (String diskId : onDisk.getStringKeySet()){
+
+                	if (!used.contains(diskId)) {
                         BibtexString disk = onDisk.getString(diskId);
 
                         if (disk.getContent().equals(tmp.getContent())) {
@@ -357,15 +359,14 @@ public class ChangeScanner extends Thread {
 
                             // Try to find the matching one in memory:
                             BibtexString bsMem = null;
-                            findInMem: for (Iterator k=inMem.getStringKeySet().iterator(); k.hasNext();) {
-                                Object memId = k.next();
-                                //for (int k = 0; k < inMem.getStringCount(); k++) {
+                            
+                            for (String memId : inMem.getStringKeySet()){
                                 BibtexString bsMem_cand = inMem.getString(memId);
                                 if (bsMem_cand.getContent().equals(disk.getContent()) &&
                                 !usedInMem.contains(memId)) {
                                     usedInMem.add(memId);
                                     bsMem = bsMem_cand;
-                                    break findInMem;
+                                    break;
                                 }
                             }
 
@@ -383,8 +384,8 @@ public class ChangeScanner extends Thread {
 
         if (notMatched.size() > 0) {
             // Still one or more non-matched strings. So they must have been removed.
-            for (Iterator i = notMatched.iterator(); i.hasNext(); ) {
-                Object nmId = i.next();
+            for (Iterator<String> i = notMatched.iterator(); i.hasNext(); ) {
+                String nmId = i.next();
                 BibtexString tmp = onTmp.getString(nmId);
                 BibtexString mem = findString(inMem, tmp.getName(), usedInMem);
                 if (mem != null) { // The removed string is not removed from the mem version.
@@ -396,8 +397,8 @@ public class ChangeScanner extends Thread {
 
         // Finally, see if there are remaining strings in the disk database. They
         // must have been added.
-        for (Iterator i=onDisk.getStringKeySet().iterator(); i.hasNext();) {
-            Object diskId = i.next();
+        for (Iterator<String> i=onDisk.getStringKeySet().iterator(); i.hasNext();) {
+            String diskId = i.next();
             if (!used.contains(diskId)) {
                 BibtexString disk = onDisk.getString(diskId);
                 //System.out.println(disk.getName());
@@ -407,11 +408,11 @@ public class ChangeScanner extends Thread {
         }
     }
 
-    private BibtexString findString(BibtexDatabase base, String name, HashSet used) {
+    private BibtexString findString(BibtexDatabase base, String name, HashSet<Object> used) {
         if (!base.hasStringLabel(name))
             return null;
-        for (Iterator i=base.getStringKeySet().iterator(); i.hasNext();) {
-            Object key = i.next();
+        for (Iterator<String> i=base.getStringKeySet().iterator(); i.hasNext();) {
+            String key = i.next();
             BibtexString bs = base.getString(key);
             if (bs.getName().equals(name) && !used.contains(key)) {
                 used.add(key);
@@ -427,7 +428,6 @@ public class ChangeScanner extends Thread {
      * properly, so I rather only report the change.
      */
     public void scanGroups(MetaData inMem, MetaData onTmp, MetaData onDisk) {
-        final GroupTreeNode groupsMem = inMem.getGroups();
         final GroupTreeNode groupsTmp = onTmp.getGroups();
         final GroupTreeNode groupsDisk = onDisk.getGroups();
         if (groupsTmp == null && groupsDisk == null)
@@ -492,4 +492,8 @@ public class ChangeScanner extends Thread {
 //        }
     }
 
+
+    public static interface DisplayResultCallback {
+        public void scanResultsResolved(boolean resolved);
+    }
 }
