@@ -1,12 +1,11 @@
 package net.sf.jabref.collab;
 
+import javax.swing.JComponent;
+import javax.swing.JScrollPane;
+
+import net.sf.jabref.*;
 import net.sf.jabref.undo.NamedCompound;
 import net.sf.jabref.undo.UndoableInsertEntry;
-import javax.swing.JComponent;
-import javax.swing.JLabel;
-import net.sf.jabref.*;
-
-import javax.swing.JScrollPane;
 
 public class EntryAddChange extends Change {
 
@@ -19,19 +18,14 @@ public class EntryAddChange extends Change {
     super("Added entry");
     this.diskEntry = diskEntry;
 
-    pp = new PreviewPanel(diskEntry, new MetaData(), Globals.prefs.get("preview0"));
+    pp = new PreviewPanel(null, diskEntry, null, new MetaData(), Globals.prefs.get("preview0"));
     sp = new JScrollPane(pp);
   }
 
   public void makeChange(BasePanel panel, NamedCompound undoEdit) {
-    try {
       diskEntry.setId(Util.createNeutralId());
       panel.database().insertEntry(diskEntry);
       undoEdit.addEdit(new UndoableInsertEntry(panel.database(), diskEntry, panel));
-    } catch (KeyCollisionException ex) {
-
-    }
-
   }
 
   JComponent description() {
