@@ -1,10 +1,9 @@
 package net.sf.jabref.collab;
 
 import javax.swing.JComponent;
-import javax.swing.JLabel;
-import net.sf.jabref.*;
-
 import javax.swing.JScrollPane;
+
+import net.sf.jabref.*;
 import net.sf.jabref.undo.NamedCompound;
 import net.sf.jabref.undo.UndoableRemoveEntry;
 
@@ -23,7 +22,7 @@ public class EntryDeleteChange extends Change {
 
     // Compare the deleted entry in memory with the one in the tmpfile. The
     // entry could have been removed in memory.
-    matchWithTmp = Util.compareEntriesStrictly(memEntry, tmpEntry);
+    matchWithTmp = DuplicateCheck.compareEntriesStrictly(memEntry, tmpEntry);
 
     // Check if it has been modified locally, since last tempfile was saved.
     isModifiedLocally = !(matchWithTmp > 1);
@@ -31,7 +30,7 @@ public class EntryDeleteChange extends Change {
     //Util.pr("Modified entry: "+memEntry.getCiteKey()+"\n Modified locally: "+isModifiedLocally
     //        +" Modifications agree: "+modificationsAgree);
 
-    pp = new PreviewPanel(memEntry, new MetaData(), Globals.prefs.get("preview0"));
+    pp = new PreviewPanel(null, memEntry, null, new MetaData(), Globals.prefs.get("preview0"));
     sp = new JScrollPane(pp);
   }
 
