@@ -4,9 +4,9 @@
 //  Language: Java
 //  Compiler: JDK 1.4
 //  Authors:  Joerg K. Wegner
-//  Version:  $Revision: 1296 $
-//            $Date: 2005-10-23 05:36:09 +0200 (Sun, 23 Oct 2005) $
-//            $Author: mortenalver $
+//  Version:  $Revision: 2268 $
+//            $Date: 2007-08-20 01:37:05 +0200 (Mon, 20 Aug 2007) $
+//            $Author: coezbek $
 //
 //  Copyright (c) Dept. Computer Architecture, University of Tuebingen, Germany
 //
@@ -33,7 +33,7 @@ import net.sf.jabref.AuthorList;
  * Create DocBook authors formatter.
  *
  * @author $author$
- * @version $Revision: 1296 $
+ * @version $Revision: 2268 $
  */
 public class CreateDocBookAuthors implements LayoutFormatter
 {
@@ -46,11 +46,7 @@ public class CreateDocBookAuthors implements LayoutFormatter
         //     <author><firstname>J.</firstname><othername role="mi">W.</othername><surname>Godden</surname></author>
         //     <author><firstname>J.</firstname><surname>Bajorath</surname></author>
 
-        int index = 0;
-        int oldPos = 0;
-        String author;
         StringBuffer sb = new StringBuffer(100);
-        //fieldText = (new ConvertSpecialCharactersForXML()).format(fieldText);
 
         if (fieldText.indexOf(" and ") == -1)
         {
@@ -85,7 +81,7 @@ public class CreateDocBookAuthors implements LayoutFormatter
     protected void singleAuthor(StringBuffer sb, String author)
     {
         // TODO: replace special characters
-        Vector v = new Vector();
+        Vector<String> v = new Vector<String>();
         String authorMod = AuthorList.fixAuthor_firstNameFirst(author);
 
         WSITools.tokenize(v, authorMod, " \n\r");
