@@ -4,9 +4,9 @@
 //  Language: Java
 //  Compiler: JDK 1.4
 //  Authors:  Joerg K. Wegner
-//  Version:  $Revision: 742 $
-//            $Date: 2005-01-27 22:31:00 +0100 (Thu, 27 Jan 2005) $
-//            $Author: egonw $
+//  Version:  $Revision: 2268 $
+//            $Date: 2007-08-20 01:37:05 +0200 (Mon, 20 Aug 2007) $
+//            $Author: coezbek $
 //
 //  Copyright (c) Dept. Computer Architecture, University of Tuebingen, Germany
 //
@@ -21,19 +21,13 @@
 ///////////////////////////////////////////////////////////////////////////////
 package net.sf.jabref.export.layout.format;
 
-import wsi.ra.tool.WSITools;
-
-import java.util.Vector;
-
-import net.sf.jabref.export.layout.LayoutFormatter;
-import net.sf.jabref.imports.*;
 
 
 /**
  * Create DocBook editors formatter.
  *
  * @author $author$
- * @version $Revision: 742 $
+ * @version $Revision: 2268 $
  */
 public class CreateDocBookEditors extends CreateDocBookAuthors
 {
@@ -43,9 +37,6 @@ public class CreateDocBookEditors extends CreateDocBookAuthors
     {
         //		<editor><firstname>L.</firstname><surname>Xue</surname></editor>
 
-        int index = 0;
-        int oldPos = 0;
-        String author;
         StringBuffer sb = new StringBuffer(100);
         //fieldText = (new ConvertSpecialCharactersForXML()).format(fieldText);
 
