@@ -5,12 +5,7 @@ import java.io.IOException;
 import java.util.Iterator;
 import java.util.List;
 
-import javax.swing.ButtonGroup;
-import javax.swing.JCheckBox;
-import javax.swing.JLabel;
-import javax.swing.JOptionPane;
-import javax.swing.JPanel;
-import javax.swing.JRadioButton;
+import javax.swing.*;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 
@@ -162,7 +157,7 @@ public class DroppedFileHandler {
             return false;
         }
 
-        List xmpEntriesInFile = null;
+        List<BibtexEntry> xmpEntriesInFile = null;
         try {
             xmpEntriesInFile = XMPUtil.readXMP(fileName);
         } catch (Exception e) {
@@ -201,7 +196,7 @@ public class DroppedFileHandler {
          */
 
         boolean isSingle = xmpEntriesInFile.size() == 1;
-        BibtexEntry single = (isSingle ? (BibtexEntry) xmpEntriesInFile.get(0) : null);
+        BibtexEntry single = (isSingle ? xmpEntriesInFile.get(0) : null);
 
        
         boolean success = true;
@@ -225,11 +220,11 @@ public class DroppedFileHandler {
         }
         if (success) {
 
-            Iterator it = xmpEntriesInFile.iterator();
+            Iterator<BibtexEntry> it = xmpEntriesInFile.iterator();
 
             while (it.hasNext()) {
                 try {
-                    BibtexEntry entry = (BibtexEntry) it.next();
+                    BibtexEntry entry = it.next();
                     entry.setId(Util.createNeutralId());
                     edits.addEdit(new UndoableInsertEntry(panel.getDatabase(), entry, panel));
                     panel.getDatabase().insertEntry(entry);
@@ -310,10 +305,10 @@ public class DroppedFileHandler {
         boolean avoidDuplicate, NamedCompound edits) {
 
 
-        Object oldValue = entry.getField(GUIGlobals.FILE_FIELD);
+        String oldValue = entry.getField(GUIGlobals.FILE_FIELD);
         FileListTableModel tm = new FileListTableModel();
         if (oldValue != null)
-            tm.setContent((String)oldValue);
+            tm.setContent(oldValue);
 
         // If avoidDuplicate==true, we should check if this file is already linked:
         if (avoidDuplicate) {
