@@ -1,29 +1,42 @@
 package net.sf.jabref.external;
 
+import java.io.BufferedWriter;
+import java.io.File;
+import java.io.FileWriter;
+import java.io.IOException;
+
+import javax.swing.*;
+
 import net.sf.jabref.*;
 
-import javax.swing.*;
-import java.io.*;
-import java.awt.event.*;
-
 public class PushToLyx implements PushToApplication {
 
+    private JTextField lyxPipe=new JTextField(30);
+    private JPanel settings = null;
+
     private boolean couldNotFindPipe=false;
     private boolean couldNotWrite=false;
     private String message = "";
 
-    public void pushEntries(final BibtexEntry[] entries, final String keyString) {
+    public void pushEntries(BibtexDatabase database, final BibtexEntry[] entries, final String keyString, MetaData metaData) {
 
         couldNotFindPipe = false;
         couldNotWrite = false;
 
-
-        final File lyxpipe = new File( Globals.prefs.get("lyxpipe") +".in"); // this needs to fixed because it gives "asdf" when going prefs.get("lyxpipe")
-        if( !lyxpipe.exists() || !lyxpipe.canWrite()){
-            couldNotFindPipe = true;
-            return;
+        String lyxpipeSetting = Globals.prefs.get("lyxpipe");
+        if (!lyxpipeSetting.endsWith(".in"))
+            lyxpipeSetting = lyxpipeSetting+".in";
+        File lp = new File(lyxpipeSetting); // this needs to fixed because it gives "asdf" when going prefs.get("lyxpipe")
+        if( !lp.exists() || !lp.canWrite()){
+            // See if it helps to append ".in":
+            lp = new File(lyxpipeSetting+".in");
+            if( !lp.exists() || !lp.canWrite()){
+                couldNotFindPipe = true;
+                return;
+            }
         }
 
+        final File lyxpipe = lp;
         Thread t = new Thread(new Runnable() {
             public void run() {
                 try {
@@ -94,6 +107,22 @@ public class PushToLyx implements PushToApplication {
         return true;
     }
 
+    public JPanel getSettingsPanel() {
+        if (settings == null)
+            initSettingsPanel();
+        lyxPipe.setText(Globals.prefs.get("lyxpipe"));
+        return settings;
+    }
+
+    public void storeSettings() {
+        Globals.prefs.put("lyxpipe", lyxPipe.getText());
+    }
+
+    private void initSettingsPanel() {
+        settings = new JPanel();
+        settings.add(new JLabel(Globals.lang("Path to LyX pipe") + ":"));
+        settings.add(lyxPipe);
+    }
     /*class Timeout extends javax.swing.Timer
     {
       public Timeout(int timeout, final Thread toStop, final String message) {
