@@ -1,6 +1,5 @@
 package net.sf.jabref.external;
 
-import net.sf.jabref.GUIGlobals;
 
 /**
  * This subclass of ExternalFileType is used to mark types that are unknown.
@@ -10,7 +9,7 @@ import net.sf.jabref.GUIGlobals;
 public class UnknownExternalFileType extends ExternalFileType {
 
     public UnknownExternalFileType(String name) {
-        super(name, "", "", "unknown");
+        super(name, "", "", "", "unknown");
     }
     
 }
