@@ -26,13 +26,16 @@ import java.util.Map;
 
 import javax.swing.undo.AbstractUndoableEdit;
 
-import net.sf.jabref.*;
+import net.sf.jabref.BibtexDatabase;
+import net.sf.jabref.BibtexEntry;
+import net.sf.jabref.SearchRule;
 
 /**
  * A group of BibtexEntries.
  */
 public abstract class AbstractGroup {
-	/** The group's name (every type of group has one). */
+
+    /** The group's name (every type of group has one). */
 	protected String m_name;
 
 	/**
@@ -42,7 +45,9 @@ public abstract class AbstractGroup {
 	 */
 	protected int m_context = INDEPENDENT;
 
-	public AbstractGroup(String name, int context) {
+    public abstract String getTypeId();
+
+    public AbstractGroup(String name, int context) {
 		m_name = name;
 		setHierarchicalContext(context);
 	}
@@ -142,7 +147,7 @@ public abstract class AbstractGroup {
 	 *            The search options to apply.
 	 * @return true if this group contains the specified entry, false otherwise.
 	 */
-	public abstract boolean contains(Map searchOptions, BibtexEntry entry);
+	public abstract boolean contains(Map<String, String> searchOptions, BibtexEntry entry);
 
 	/**
 	 * @return true if this group contains the specified entry, false otherwise.
