@@ -22,15 +22,20 @@
 
 package net.sf.jabref.groups;
 
-import java.awt.datatransfer.*;
+import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
+import java.awt.datatransfer.UnsupportedFlavorException;
 import java.io.IOException;
-import java.util.*;
+import java.util.Enumeration;
+import java.util.Vector;
 
-import javax.swing.tree.*;
+import javax.swing.tree.DefaultMutableTreeNode;
+import javax.swing.tree.TreeNode;
 import javax.swing.undo.AbstractUndoableEdit;
 
-import net.sf.jabref.*;
+import net.sf.jabref.BibtexDatabase;
+import net.sf.jabref.BibtexEntry;
+import net.sf.jabref.SearchRule;
 
 /**
  * A node in the groups tree that holds exactly one AbstractGroup.
@@ -83,10 +88,10 @@ public class GroupTreeNode extends DefaultMutableTreeNode implements
 	 */
 	public String getTreeAsString() {
 		StringBuffer sb = new StringBuffer();
-		Enumeration e = preorderEnumeration();
+		Enumeration<GroupTreeNode> e = preorderEnumeration();
 		GroupTreeNode cursor;
 		while (e.hasMoreElements()) {
-			cursor = (GroupTreeNode) e.nextElement();
+			cursor = e.nextElement();
             sb.append(cursor.getLevel()).append(" ").append(cursor.getGroup().toString()).append("\n");
 		}
 		return sb.toString();
@@ -195,23 +200,47 @@ public class GroupTreeNode extends DefaultMutableTreeNode implements
 		return searchRule;
 	}
 
+	@Override
+	@SuppressWarnings("unchecked")
+	public Enumeration<GroupTreeNode> preorderEnumeration(){
+		return super.preorderEnumeration();
+	}
+	
+	@Override
+	@SuppressWarnings("unchecked")
+	public Enumeration<GroupTreeNode> depthFirstEnumeration(){
+		return super.depthFirstEnumeration();
+	}
+	
+	@Override
+	@SuppressWarnings("unchecked")
+	public Enumeration<GroupTreeNode> breadthFirstEnumeration(){
+		return super.breadthFirstEnumeration();
+	}
+	
+	@Override
+	@SuppressWarnings("unchecked")
+	public Enumeration<GroupTreeNode> children(){
+		return super.children();
+	}
+	
 	/**
 	 * Scans the subtree rooted at this node.
 	 * 
 	 * @return All groups that contain the specified entry.
 	 */
 	public AbstractGroup[] getMatchingGroups(BibtexEntry entry) {
-		Vector matchingGroups = new Vector();
-		Enumeration e = preorderEnumeration();
+		Vector<AbstractGroup> matchingGroups = new Vector<AbstractGroup>();
+		Enumeration<GroupTreeNode> e = preorderEnumeration();
 		AbstractGroup group;
 		while (e.hasMoreElements()) {
-			group = ((GroupTreeNode) e.nextElement()).getGroup();
+			group = (e.nextElement()).getGroup();
 			if (group.contains(null, entry)) // first argument is never used
 				matchingGroups.add(group);
 		}
 		AbstractGroup[] matchingGroupsArray = new AbstractGroup[matchingGroups
 				.size()];
-		return (AbstractGroup[]) matchingGroups.toArray(matchingGroupsArray);
+		return matchingGroups.toArray(matchingGroupsArray);
 	}
 
 	public boolean canMoveUp() {
