@@ -1,23 +1,49 @@
 package net.sf.jabref.gui;
 
-import net.sf.jabref.*;
-import net.sf.jabref.groups.EntryTableTransferHandler;
-import net.sf.jabref.undo.NamedCompound;
-import net.sf.jabref.undo.UndoableFieldChange;
-import net.sf.jabref.external.*;
-
-import javax.swing.*;
-import java.awt.*;
-import java.awt.dnd.DnDConstants;
-import java.awt.datatransfer.Transferable;
+import java.awt.BorderLayout;
+import java.awt.Color;
+import java.awt.Insets;
 import java.awt.datatransfer.DataFlavor;
+import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
-import java.awt.event.*;
-import java.util.*;
-import java.util.List;
+import java.awt.dnd.DnDConstants;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.MouseAdapter;
+import java.awt.event.MouseEvent;
 import java.io.File;
 import java.io.IOException;
 import java.net.URL;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
+import javax.swing.AbstractAction;
+import javax.swing.BorderFactory;
+import javax.swing.JButton;
+import javax.swing.JComponent;
+import javax.swing.JDialog;
+import javax.swing.JLabel;
+import javax.swing.JMenuItem;
+import javax.swing.JOptionPane;
+import javax.swing.JPanel;
+import javax.swing.JPopupMenu;
+import javax.swing.JProgressBar;
+import javax.swing.JScrollPane;
+import javax.swing.JTable;
+import javax.swing.KeyStroke;
+import javax.swing.SwingUtilities;
+import javax.swing.TransferHandler;
+import javax.swing.event.UndoableEditListener;
+
+import net.sf.jabref.*;
+import net.sf.jabref.external.*;
+import net.sf.jabref.groups.EntryTableTransferHandler;
+import net.sf.jabref.undo.NamedCompound;
+import net.sf.jabref.undo.UndoableFieldChange;
 
 import com.jgoodies.forms.builder.DefaultFormBuilder;
 import com.jgoodies.forms.layout.FormLayout;
@@ -39,9 +65,10 @@ public class FileListEditor extends JTable implements FieldEditor,
     private JScrollPane sPane;
     private JButton add, remove, up, down, auto, download;
     private JPopupMenu menu = new JPopupMenu();
-    private JMenuItem item = new JMenuItem(Globals.lang("Open"));
-
 
+    private JMenuItem openLink = new JMenuItem(Globals.lang("Open"));
+    private JMenuItem rename = new JMenuItem(Globals.lang("Move/rename file"));
+    private JMenuItem moveToFileDir = new JMenuItem(Globals.lang("Move to file directory"));
 
     public FileListEditor(JabRefFrame frame, MetaData metaData, String fieldName, String content,
                           EntryEditor entryEditor) {
@@ -138,12 +165,17 @@ public class FileListEditor extends JTable implements FieldEditor,
             }
         });
 
-        menu.add(item);
-        item.addActionListener(new ActionListener() {
+        menu.add(openLink);
+        openLink.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent actionEvent) {
                 openSelectedFile();
             }
         });
+        menu.add(rename);
+        rename.addActionListener(new MoveFileAction(frame, entryEditor, this, false));
+
+        menu.add(moveToFileDir);
+        moveToFileDir.addActionListener(new MoveFileAction(frame, entryEditor, this, true));
     }
 
     private void openSelectedFile() {
@@ -158,6 +190,9 @@ public class FileListEditor extends JTable implements FieldEditor,
         }
     }
 
+    public FileListTableModel getTableModel() {
+        return tableModel;
+    }
 
     public String getFieldName() {
         return fieldName;
@@ -216,7 +251,7 @@ public class FileListEditor extends JTable implements FieldEditor,
         if (row == -1)
             row = 0;
         FileListEntry entry = new FileListEntry("", initialLink, null);
-        if (editListEntry(entry))
+        if (editListEntry(entry, true))
             tableModel.addEntry(row, entry);
         entryEditor.updateField(this);
     }
@@ -225,21 +260,6 @@ public class FileListEditor extends JTable implements FieldEditor,
         addEntry("");
     }
 
-    /**
-     * Add the listed files, prompting the user with the entry editor in each case.
-     * @param files A list of File objects.
-     */
-    private void addAll(List files) {
-        for (Iterator i = files.iterator(); i.hasNext();) {
-            File file = (File)i.next();
-            String fileName = file.getAbsolutePath();
-            FileListEntry entry = new FileListEntry("", fileName, null);
-            if (editListEntry(entry))
-                    tableModel.addEntry(tableModel.getRowCount(), entry);
-        }
-        entryEditor.updateField(this);
-    }
-
     private void removeEntries() {
         int[] rows = getSelectedRows();
         if (rows != null)
@@ -265,13 +285,19 @@ public class FileListEditor extends JTable implements FieldEditor,
         setRowSelectionInterval(toIdx, toIdx);
     }
 
-    private boolean editListEntry(FileListEntry entry) {
+    /**
+     * Open an editor for this entry.
+     * @param entry The entry to edit.
+     * @param openBrowse True to indicate that a Browse dialog should be immediately opened.
+     * @return true if the edit was accepted, false if it was cancelled.
+     */
+    private boolean editListEntry(FileListEntry entry, boolean openBrowse) {
         if (editor == null) {
             editor = new FileListEntryEditor(frame, entry, false, true, metaData);
         }
         else
             editor.setEntry(entry);
-        editor.setVisible(true);
+        editor.setVisible(true, openBrowse);
         if (editor.okPressed())
             tableModel.fireTableDataChanged();
         entryEditor.updateField(this);
@@ -281,7 +307,6 @@ public class FileListEditor extends JTable implements FieldEditor,
     private void autoSetLinks() {
         auto.setEnabled(false);
         BibtexEntry entry = entryEditor.getEntry();
-        int tableSize = tableModel.getRowCount();
         JDialog diag = new JDialog(frame, true);
         autoSetLinks(entry, tableModel, metaData, new ActionListener() {
             public void actionPerformed(ActionEvent e) {
@@ -316,12 +341,9 @@ public class FileListEditor extends JTable implements FieldEditor,
                                       final Set<BibtexEntry> changedEntries,
                                       final ArrayList<File> dirs) {
 
-        final ExternalFileType[] types = Globals.prefs.getExternalFileTypeSelection();
-        final JLabel label = new JLabel(Globals.lang("Searching for files"));
         Runnable r = new Runnable() {
 
             public void run() {
-                boolean foundAny = false;
                 ExternalFileType[] types = Globals.prefs.getExternalFileTypeSelection();
                 Collection<String> extensions = new ArrayList<String>();
                 for (int i = 0; i < types.length; i++) {
@@ -329,25 +351,31 @@ public class FileListEditor extends JTable implements FieldEditor,
                     extensions.add(type.getExtension());
                 }
                 // Run the search operation:
-                Map<BibtexEntry, java.util.List<File>> result =
-                        Util.findAssociatedFiles(entries, extensions, dirs);
+                Map<BibtexEntry, java.util.List<File>> result;
+                if (Globals.prefs.getBoolean(JabRefPreferences.USE_REG_EXP_SEARCH_KEY)) {
+                    String regExp = Globals.prefs.get(JabRefPreferences.REG_EXP_SEARCH_EXPRESSION_KEY);
+                    result = RegExpFileSearch.findFilesForSet(entries, extensions, dirs, regExp);
+                }
+                else
+                    result = Util.findAssociatedFiles(entries, extensions, dirs);
+
 
                 // Iterate over the entries:
                 for (Iterator<BibtexEntry> i=result.keySet().iterator(); i.hasNext();) {
                     BibtexEntry anEntry = i.next();
                     FileListTableModel tableModel = new FileListTableModel();
-                    Object oldVal = anEntry.getField(GUIGlobals.FILE_FIELD);
+                    String oldVal = anEntry.getField(GUIGlobals.FILE_FIELD);
                     if (oldVal != null)
-                        tableModel.setContent((String)oldVal);
+                        tableModel.setContent(oldVal);
                     List<File> files = result.get(anEntry);
                     for (File f : files) {
-			f = relativizePath(f, dirs);
+			            f = relativizePath(f, dirs);
                         boolean alreadyHas = false;
-			//System.out.println("File: "+f.getPath());
+			            //System.out.println("File: "+f.getPath());
                         for (int j = 0; j < tableModel.getRowCount(); j++) {
                             FileListEntry existingEntry = tableModel.getEntry(j);
-			    //System.out.println("Comp: "+existingEntry.getLink());
-			    if (new File(existingEntry.getLink()).equals(f)) {
+                            //System.out.println("Comp: "+existingEntry.getLink());
+                            if (new File(existingEntry.getLink()).equals(f)) {
                                 alreadyHas = true;
                                 break;
                             }
@@ -356,7 +384,7 @@ public class FileListEditor extends JTable implements FieldEditor,
                             int index = f.getPath().lastIndexOf('.');
                             if ((index >= 0) && (index < f.getPath().length()-1)) {
                                 ExternalFileType type = Globals.prefs.getExternalFileTypeByExt
-                                    (f.getPath().substring(index+1));
+                                    (f.getPath().substring(index+1).toLowerCase());
                                 FileListEntry flEntry = new FileListEntry(f.getName(), f.getPath(), type);
                                 tableModel.addEntry(tableModel.getRowCount(), flEntry);
                             } else {
@@ -433,8 +461,13 @@ public class FileListEditor extends JTable implements FieldEditor,
                     extensions.add(type.getExtension());
                 }
                 // Run the search operation:
-                Map<BibtexEntry, java.util.List<File>> result =
-                        Util.findAssociatedFiles(entries, extensions, dirs);
+                Map<BibtexEntry, java.util.List<File>> result;
+                if (Globals.prefs.getBoolean(JabRefPreferences.USE_REG_EXP_SEARCH_KEY)) {
+                    String regExp = Globals.prefs.get(JabRefPreferences.REG_EXP_SEARCH_EXPRESSION_KEY);
+                    result = RegExpFileSearch.findFilesForSet(entries, extensions, dirs, regExp);
+                }
+                else
+                    result = Util.findAssociatedFiles(entries, extensions, dirs);
 
                 // Iterate over the entries:
                 for (Iterator<BibtexEntry> i=result.keySet().iterator(); i.hasNext();) {
@@ -547,7 +580,7 @@ public class FileListEditor extends JTable implements FieldEditor,
                 int row = rowAtPoint(e.getPoint());
                 if (row >= 0) {
                     FileListEntry entry = tableModel.getEntry(row);
-                    editListEntry(entry);
+                    editListEntry(entry, false);
                 }
             }
             else if (e.isPopupTrigger())
@@ -603,22 +636,19 @@ public class FileListEditor extends JTable implements FieldEditor,
             return importData(FileListEditor.this, transferSupport.getTransferable());
         }*/
 
+        @SuppressWarnings("unchecked")
         public boolean importData(JComponent comp, Transferable t) {
             // If the drop target is the main table, we want to record which
             // row the item was dropped on, to identify the entry if needed:
-            int dropRow = -1;
-            if (comp instanceof JTable) {
-                dropRow = ((JTable) comp).getSelectedRow();
-            }
 
             try {
 		
-                List files = null;
+                List<File> files = null;
                 // This flavor is used for dragged file links in Windows:
                 if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                     // JOptionPane.showMessageDialog(null, "Received
                     // javaFileListFlavor");
-                    files = (List) t.getTransferData(DataFlavor.javaFileListFlavor);
+                    files = (List<File>) t.getTransferData(DataFlavor.javaFileListFlavor);
                 }
 
                 if (t.isDataFlavorSupported(urlFlavor)) {
@@ -635,12 +665,11 @@ public class FileListEditor extends JTable implements FieldEditor,
                 }
 
 		        if (files != null) {
-		            final List theFiles = files;
+		            final List<File> theFiles = files;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
                             //addAll(files);
-                            for (Iterator i=theFiles.iterator(); i.hasNext();) {
-			                    File f = (File)i.next();
+                            for (File f : theFiles){
                                 // Find the file's extension, if any:
                                 String name = f.getAbsolutePath();
                                 String extension = "";
@@ -698,4 +727,21 @@ public class FileListEditor extends JTable implements FieldEditor,
         }
 
     }
+
+    public boolean hasUndoInformation() {
+        return false;
+    }
+
+    public void undo() {
+    }
+
+    public boolean hasRedoInformation() {
+        return false;
+    }
+
+    public void redo() {
+    }
+
+    public void addUndoableEditListener(UndoableEditListener listener) {
+    }    
 }
