@@ -1,23 +1,38 @@
 package net.sf.jabref.gui;
 
-import ca.odell.glazedlists.event.ListEventListener;
-import ca.odell.glazedlists.event.ListEvent;
-import ca.odell.glazedlists.EventList;
-import ca.odell.glazedlists.SortedList;
-import net.sf.jabref.*;
-import net.sf.jabref.external.ExternalFileMenuItem;
-
-import javax.swing.*;
-import java.awt.event.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.FocusEvent;
+import java.awt.event.FocusListener;
+import java.awt.event.KeyEvent;
+import java.awt.event.KeyListener;
+import java.awt.event.MouseEvent;
+import java.awt.event.MouseListener;
 import java.io.IOException;
-import java.util.Comparator;
-import java.util.List;
+
+import javax.swing.JPopupMenu;
+import javax.swing.SwingUtilities;
+import javax.swing.Timer;
+
+import net.sf.jabref.BasePanel;
+import net.sf.jabref.BibtexEntry;
+import net.sf.jabref.EntryEditor;
+import net.sf.jabref.FocusRequester;
+import net.sf.jabref.GUIGlobals;
+import net.sf.jabref.Globals;
+import net.sf.jabref.PreviewPanel;
+import net.sf.jabref.RightClickMenu;
+import net.sf.jabref.Util;
+import net.sf.jabref.external.ExternalFileMenuItem;
+import ca.odell.glazedlists.EventList;
+import ca.odell.glazedlists.event.ListEvent;
+import ca.odell.glazedlists.event.ListEventListener;
 
 /**
  * List event, mouse, key and focus listener for the main table that makes up the
  * most part of the BasePanel for a single bib database.
  */
-public class MainTableSelectionListener implements ListEventListener, MouseListener,
+public class MainTableSelectionListener implements ListEventListener<BibtexEntry>, MouseListener,
         KeyListener, FocusListener {
 
     PreviewPanel[] previewPanel = null;
@@ -25,7 +40,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
     PreviewPanel preview;
     MainTable table;
     BasePanel panel;
-    EventList tableRows;
+    EventList<BibtexEntry> tableRows;
     private boolean previewActive = Globals.prefs.getBoolean("previewEnabled");
     private boolean workingOnPreview = false;
 
@@ -34,7 +49,6 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
     // key strokes cycle between all entries starting with the same letter:
     private int[] lastPressed = new int[20];
     private int lastPressedCount = 0;
-    private int lastQuickJumpRow = -1;
     private long lastPressedTime = 0;
     private long QUICK_JUMP_TIMEOUT = 2000;
 
@@ -49,13 +63,15 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
     }
 
     private void instantiatePreviews() {
-        previewPanel = new PreviewPanel[]
-                {new PreviewPanel(panel.database(), panel.metaData(), Globals.prefs.get("preview0")),
-                        new PreviewPanel(panel.database(), panel.metaData(), Globals.prefs.get("preview1"))};
-        BibtexEntry testEntry = PreviewPrefsTab.getTestEntry();
-        previewPanel[0].setEntry(testEntry);
-        previewPanel[1].setEntry(testEntry);
-    }
+		previewPanel = new PreviewPanel[] {
+			new PreviewPanel(panel.database(), null, panel, panel.metaData(), Globals.prefs
+				.get("preview0")),
+			new PreviewPanel(panel.database(), null, panel, panel.metaData(), Globals.prefs
+				.get("preview1")) };
+		//BibtexEntry testEntry = PreviewPrefsTab.getTestEntry();
+		//previewPanel[0].setEntry(testEntry);
+		//previewPanel[1].setEntry(testEntry);
+	}
 
     public void updatePreviews() {
         try {
@@ -66,9 +82,8 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
         }
     }
 
-    public void listChanged(ListEvent e) {
-        //System.out.println(e);
-        EventList selected = e.getSourceList();
+    public void listChanged(ListEvent<BibtexEntry> e) {
+        EventList<BibtexEntry> selected = e.getSourceList();
         Object newSelected = null;
         while (e.next()) {
             if (e.getType() == ListEvent.INSERT) {
@@ -96,6 +111,12 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
                 // Show the new editor unless it was already visible:
                 if ((newEditor != oldEditor) || (mode != BasePanel.SHOWING_EDITOR)) {
                     panel.showEntryEditor(newEditor);
+                    SwingUtilities.invokeLater(new Runnable() {
+                        public void run() {
+                            table.ensureVisible(table.getSelectedRow());
+                        }
+                    });
+
                 }
             } else {
                 // Either nothing or a preview was shown. Update the preview.
@@ -125,9 +146,9 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
             t.start();
             return;
         }
-        EventList list = table.getSelected();
+        EventList<BibtexEntry> list = table.getSelected();
         // Check if the entry to preview is still selected:
-        if ((list.size() != 1) || ((BibtexEntry)list.get(0) != toShow)) {
+        if ((list.size() != 1) || (list.get(0) != toShow)) {
             return;
         }
         final int mode = panel.getMode();
@@ -153,7 +174,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
 
     public void editSignalled() {
         if (table.getSelected().size() == 1) {
-            editSignalled((BibtexEntry) table.getSelected().get(0));
+            editSignalled(table.getSelected().get(0));
         }
     }
 
@@ -168,12 +189,13 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
     }
 
     public void mouseReleased(MouseEvent e) {
-        // First find the column on which the user has clicked.
-        final int col = table.columnAtPoint(e.getPoint()),
-                row = table.rowAtPoint(e.getPoint());
+        // First find the row on which the user has clicked.
+        final int row = table.rowAtPoint(e.getPoint());
+
 
         // Check if the user has right-clicked. If so, open the right-click menu.
         if (e.isPopupTrigger()) {
+            final int col = table.columnAtPoint(e.getPoint());
             // Check if the user has clicked on an icon cell to open url or pdf.
             final String[] iconType = table.getIconTypeForColumn(col);
             
@@ -193,7 +215,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
         // A double click on an entry should open the entry's editor.
         if (e.getClickCount() == 2) {
 
-            BibtexEntry toShow = (BibtexEntry) tableRows.get(row);
+            BibtexEntry toShow = tableRows.get(row);
             editSignalled(toShow);
         }
 
@@ -222,7 +244,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
             Object value = table.getValueAt(row, col);
             if (value == null) return; // No icon here, so we do nothing.
 
-            final BibtexEntry entry = (BibtexEntry) tableRows.get(row);
+            final BibtexEntry entry = tableRows.get(row);
 
             // Get the icon type. Corresponds to the field name.
             int hasField = -1;
@@ -239,7 +261,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
                     panel.output(Globals.lang("External viewer called") + ".");
 
                     Object link = entry.getField(fieldName);
-                    if (iconType == null) {
+                    if (link == null) {
                         Globals.logger("Error: no link to " + fieldName + ".");
                         return; // There is an icon, but the field is not set.
                     }
@@ -259,7 +281,10 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
                                         (panel.frame(), entry, "",
                                         flEntry.getLink(), flEntry.getType().getIcon(),
                                         panel.metaData(), flEntry.getType());
-                                item.actionPerformed(null);
+                                boolean success = item.openLink();
+                                if (!success) {
+                                    panel.output(Globals.lang("Unable to open link."));
+                                }
                             }
                         } else {
                             Util.openExternalViewer(panel.metaData(), (String)link, fieldName);
@@ -303,7 +328,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
      *  this table cell.
      */
     private void showIconRightClickMenu(MouseEvent e, int row, String[] iconType) {
-        BibtexEntry entry = (BibtexEntry) tableRows.get(row);
+        BibtexEntry entry = tableRows.get(row);
         JPopupMenu menu = new JPopupMenu();
         int count = 0;
 
@@ -333,7 +358,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
                 if (o != null) {
                     menu.add(new ExternalFileMenuItem(panel.frame(), entry, (String)o, (String)o,
                             GUIGlobals.getTableIcon(iconType[i]).getIcon(),
-                            panel.metaData()));
+                            panel.metaData(), iconType[i]));
                     count++;
                 }
             }
@@ -374,7 +399,7 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
             panel.hideBottomComponent();
         } else {
             if (table.getSelected().size() > 0 ) {
-                updatePreview((BibtexEntry) table.getSelected().get(0), false);
+                updatePreview(table.getSelected().get(0), false);
             }
         }
     }
@@ -388,43 +413,11 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
             this.preview = previewPanel[activePreview];
 
             if (table.getSelected().size() > 0) {
-                updatePreview((BibtexEntry) table.getSelected().get(0), true);
+                updatePreview(table.getSelected().get(0), true);
             }
         }
     }
 
-    /**
-     * We should implement a faster method than the one below, but this one doesn't
-     * work currently.
-     * @param e The KeyEvent
-     */
-    public void keyTyped_(KeyEvent e) {
-        //System.out.println(e.getKeyChar()+" "+table.getSortingColumn());
-        if ((!e.isActionKey()) && Character.isLetterOrDigit(e.getKeyChar())) {
-            int sortingColumn = table.getSortingColumn(0);
-            if (sortingColumn < 0)
-                return;
-            Comparator comp = table.getComparatorForColumn(sortingColumn);
-            int piv = 1;
-            while (((sortingColumn = table.getSortingColumn(piv)) >= 0)
-                && ((comp = table.getComparatorForColumn(sortingColumn)) != null)
-                && !(comp instanceof FieldComparator)) {
-                piv++;
-            }
-            if ((comp == null) || !(comp instanceof FieldComparator))
-                return;
-
-            // Ok, after all of this we should have either found a field name to go by
-            String field = ((FieldComparator)comp).getFieldName();
-            System.out.println(String.valueOf(e.getKeyChar())+" "+field);
-            SortedList list = table.getSortedForTable();
-            BibtexEntry testEntry = new BibtexEntry("0");
-            testEntry.setField(field, String.valueOf(e.getKeyChar()));
-            int i = list.sortIndex(testEntry);
-            System.out.println(i);
-        }
-    }
-
     /**
      * Receive key event on the main table. If the key is a letter or a digit,
      * we should select the first entry in the table which starts with the given
@@ -472,7 +465,6 @@ public class MainTableSelectionListener implements ListEventListener, MouseListe
                                 // We found a match:
                                 table.setRowSelectionInterval(i, i);
                                 table.ensureVisible(i);
-                                lastQuickJumpRow = i;
                                 return;
                             }
                         }
