@@ -1,17 +1,14 @@
 package net.sf.jabref.imports;
 
-import java.util.regex.Pattern;
-import java.io.InputStream;
 import java.io.BufferedReader;
 import java.io.IOException;
-import java.util.List;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.HashMap;
-import net.sf.jabref.BibtexEntry;
-import net.sf.jabref.Globals;
-import net.sf.jabref.Util;
-import net.sf.jabref.AuthorList;
-import net.sf.jabref.BibtexFields;
+import java.util.List;
+import java.util.regex.Pattern;
+
+import net.sf.jabref.*;
 
 /**
  * Importer for the Refer/Endnote format.
@@ -59,8 +56,8 @@ public class EndnoteImporter extends ImportFormat {
      * Parse the entries in the source, and return a List of BibtexEntry
      * objects.
      */
-    public List importEntries(InputStream stream) throws IOException {
-    ArrayList bibitems = new ArrayList();
+    public List<BibtexEntry> importEntries(InputStream stream) throws IOException {
+    ArrayList<BibtexEntry> bibitems = new ArrayList<BibtexEntry>();
     StringBuffer sb = new StringBuffer();
     BufferedReader in = new BufferedReader(ImportFormatReader.getReaderDefaultEncoding(stream));
     String ENDOFRECORD = "__EOREOR__";
@@ -82,7 +79,7 @@ public class EndnoteImporter extends ImportFormat {
     }
 
     String[] entries = sb.toString().split(ENDOFRECORD);
-    HashMap hm = new HashMap();
+    HashMap<String, String> hm = new HashMap<String, String>();
     String author = "", Type = "", editor = "", artnum = "";
     for (int i = 0; i < entries.length; i++){
         hm.clear();
@@ -90,6 +87,7 @@ public class EndnoteImporter extends ImportFormat {
         Type = "";
         editor = "";
         artnum = "";
+
         boolean IsEditedBook = false;
         String[] fields = entries[i].trim().substring(1).split("\n%");
         //String lastPrefix = "";
@@ -164,6 +162,7 @@ public class EndnoteImporter extends ImportFormat {
         else if (prefix.equals("V")) hm.put("volume", val);
         else if (prefix.equals("N")) hm.put("number", val);
         else if (prefix.equals("U")) hm.put("url", val);
+        else if (prefix.equals("R")) hm.put("doi", val);
         else if (prefix.equals("O")) {
 	    // Notes may contain Article number
 	    if (val.startsWith("Artn")) {
@@ -195,7 +194,7 @@ public class EndnoteImporter extends ImportFormat {
         if (!author.equals("")) hm.put("author", fixAuthor(author));
         if (!editor.equals("")) hm.put("editor", fixAuthor(editor));
         //if pages missing and article number given, use the article number
-        if (hm.get("pages").equals("-") && !artnum.equals(""))
+        if ((hm.get("pages") == null) || hm.get("pages").equals("-") && !artnum.equals(""))
             hm.put("pages", artnum);
 
         BibtexEntry b = new BibtexEntry(BibtexFields.DEFAULT_BIBTEXENTRY_ID, Globals
@@ -203,7 +202,8 @@ public class EndnoteImporter extends ImportFormat {
         // create one here
         b.setField(hm);
         //if (hm.isEmpty())
-        if (b.getAllFields().length > 0) bibitems.add(b);
+        if (b.getAllFields().size() > 0)
+        	bibitems.add(b);
 
     }
     return bibitems;
