@@ -1,15 +1,31 @@
 package net.sf.jabref.imports;
 
-import net.sf.jabref.*;
-import net.sf.jabref.undo.NamedCompound;
-import net.sf.jabref.gui.ImportInspectionDialog;
-
-import javax.swing.*;
-import java.awt.event.ActionListener;
+import java.awt.BorderLayout;
+import java.awt.Dimension;
+import java.awt.GridBagConstraints;
+import java.awt.GridBagLayout;
+import java.awt.Insets;
 import java.awt.event.ActionEvent;
-import java.awt.*;
-import java.util.Hashtable;
-import java.util.Arrays;
+import java.awt.event.ActionListener;
+
+import javax.swing.AbstractAction;
+import javax.swing.Action;
+import javax.swing.BorderFactory;
+import javax.swing.ImageIcon;
+import javax.swing.JButton;
+import javax.swing.JPanel;
+import javax.swing.JTextField;
+
+import net.sf.jabref.BibtexFields;
+import net.sf.jabref.FocusRequester;
+import net.sf.jabref.GUIGlobals;
+import net.sf.jabref.Globals;
+import net.sf.jabref.HelpAction;
+import net.sf.jabref.JabRefFrame;
+import net.sf.jabref.SidePaneComponent;
+import net.sf.jabref.SidePaneManager;
+import net.sf.jabref.Util;
+import net.sf.jabref.gui.ImportInspectionDialog;
 
 /**
  * <p>Title: </p>
@@ -101,11 +117,22 @@ public class GeneralFetcher extends SidePaneComponent implements ActionListener
     public void actionPerformed(ActionEvent e) {
         if (tf.getText().trim().length() == 0)
             return;
-        ImportInspectionDialog dialog = new ImportInspectionDialog(frame, frame.basePanel(),
+        final ImportInspectionDialog dialog = new ImportInspectionDialog(frame, frame.basePanel(),
                 BibtexFields.DEFAULT_INSPECTION_FIELDS, fetcher.getTitle(), false);
         dialog.addCallBack(fetcher);
         Util.placeDialog(dialog, frame);
-        fetcher.processQuery(tf.getText().trim(), dialog, frame);
+        dialog.setVisible(true);
+        
+        new Thread(new Runnable(){
+            public void run(){
+                
+                if (fetcher.processQuery(tf.getText().trim(), dialog, frame)){
+                    dialog.entryListComplete();
+                } else {
+                    dialog.dispose();
+                }
+            }
+        }).start();
     }
 
     class FetcherAction extends AbstractAction {
