@@ -1,9 +1,7 @@
 package net.sf.jabref.imports;
 
-import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 import java.net.HttpURLConnection;
 import java.net.URL;
@@ -20,9 +18,8 @@ import net.sf.jabref.BibtexEntry;
 import net.sf.jabref.BibtexEntryType;
 import net.sf.jabref.GUIGlobals;
 import net.sf.jabref.Globals;
-import net.sf.jabref.JabRefFrame;
+import net.sf.jabref.OutputPrinter;
 import net.sf.jabref.Util;
-import net.sf.jabref.gui.ImportInspectionDialog;
 
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
@@ -35,14 +32,14 @@ import org.xml.sax.helpers.DefaultHandler;
  * @author Ulrich St&auml;rk
  * @author Christian Kopf
  * 
- * @version $Revision: 2456 $ ($Date: 2007-11-01 21:35:20 +0100 (Thu, 01 Nov 2007) $)
+ * @version $Revision: 2552 $ ($Date: 2007-12-04 22:58:54 +0100 (Tue, 04 Dec 2007) $)
  * 
  */
-public class OAI2Fetcher implements EntryFetcher, Runnable {
+public class OAI2Fetcher implements EntryFetcher {
 
     public static final String OAI2_ARXIV_PREFIXIDENTIFIER = "oai%3AarXiv.org%3A";
 
-    public static final String OAI2_ARXIV_HOST = "arxiv.org";
+    public static final String OAI2_ARXIV_HOST = "export.arxiv.org";
 
     public static final String OAI2_ARXIV_SCRIPT = "oai2";
 
@@ -68,14 +65,11 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
 
     private boolean shouldContinue = true;
 
-    private String query;
-
-    private ImportInspectionDialog dialog;
-
-    private JabRefFrame frame;
-
-    /* some archives - like arxive.org - might expect of you to wait some time */
+    private OutputPrinter status;
 
+    /**
+     * some archives - like arxive.org - might expect of you to wait some time 
+     */
     private boolean shouldWait() {
         return waitTime > 0;
     }
@@ -137,7 +131,7 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
     public String constructUrl(String key) {
         String identifier = "";
         try {
-            identifier = URLEncoder.encode((String) key, "UTF-8");
+            identifier = URLEncoder.encode(key, "UTF-8");
         } catch (UnsupportedEncodingException e) {
             return "";
         }
@@ -158,6 +152,11 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
      * @return Fixed key.
      */
     public static String fixKey(String key){
+        
+        if (key.toLowerCase().startsWith("arxiv:")){
+            key = key.substring(6);
+        }
+        
         int dot = key.indexOf('.');
         int slash = key.indexOf('/');
         
@@ -203,23 +202,30 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
             saxParser.parse(inputStream, handlerBase);
             
             /* Correct line breaks and spacing */
-            Object[] fields = be.getAllFields();
-            for (int i = 0; i < fields.length; i++){
-                String name = fields[i].toString();
-                
-                be.setField(name, correctLineBreaks(be.getField(name).toString()));
+            for (String name : be.getAllFields()){
+                be.setField(name, OAI2Fetcher.correctLineBreaks(be.getField(name).toString()));
             }
+            
+            if (key.matches("\\d\\d\\d\\d\\..*")){
+                be.setField("year", "20" + key.substring(0,2));
+                
+                int month = Integer.parseInt(key.substring(2,4));
+                if (month >= 1 && month <= 12){
+                    be.setField("month", "#" + Globals.MONTHS[month - 1] + "#");
+                }
+            }
+            
             return be;
         } catch (IOException e) {
-            JOptionPane.showMessageDialog(frame, Globals.lang(
+            status.showMessage(Globals.lang(
                 "An Exception ocurred while accessing '%0'", url)
                 + "\n\n" + e.toString(), Globals.lang(getKeyName()), JOptionPane.ERROR_MESSAGE);
         } catch (SAXException e) {
-            JOptionPane.showMessageDialog(frame, Globals.lang(
+            status.showMessage(Globals.lang(
                 "An SAXException ocurred while parsing '%0':", new String[]{url})
                 + "\n\n" + e.getMessage(), Globals.lang(getKeyName()), JOptionPane.ERROR_MESSAGE);
         } catch (RuntimeException e){
-            JOptionPane.showMessageDialog(frame, Globals.lang(
+            status.showMessage(Globals.lang(
                 "An Error occurred while fetching from OAI2 source (%0):", new String[]{url})
                 + "\n\n" + e.getMessage(), Globals.lang(getKeyName()), JOptionPane.ERROR_MESSAGE);
         } 
@@ -248,34 +254,19 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
         return Globals.menuTitle(getKeyName());
     }
 
-    public void processQuery(String query, ImportInspectionDialog dialog, JabRefFrame frame) {
-        this.query = query;
-        this.dialog = dialog;
-        this.frame = frame;
-        (new Thread(this)).start();
-    }
+    public boolean processQuery(String query, ImportInspector dialog, OutputPrinter status) {
 
-    public void cancelled() {
-        shouldContinue = false;
-    }
+        this.status = status;
 
-    public void done(int entriesImported) {
-        // do nothing
-    }
-
-    public void stopFetching() {
-        shouldContinue = false;
-    }
-
-    public void run() {
         try {
-            dialog.setVisible(true);
             shouldContinue = true;
+            
             /* multiple keys can be delimited by ; or space */
             query = query.replaceAll(" ", ";");
             String[] keys = query.split(";");
             for (int i = 0; i < keys.length; i++) {
                 String key = keys[i];
+                
                 /*
                  * some archives - like arxive.org - might expect of you to wait
                  * some time
@@ -285,13 +276,13 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
                     long elapsed = new Date().getTime() - lastCall.getTime();
 
                     while (elapsed < waitTime) {
-                        frame.output(Globals.lang("Waiting for ArXiv...") + ((waitTime - elapsed) / 1000) + " s");
+                        status.setStatus(Globals.lang("Waiting for ArXiv...") + ((waitTime - elapsed) / 1000) + " s");
                         Thread.sleep(1000);
                         elapsed = new Date().getTime() - lastCall.getTime();
                     }
                 }
 
-                frame.output(Globals.lang("Processing ") + key);
+                status.setStatus(Globals.lang("Processing ") + key);
 
                 /* the cancel button has been hit */
                 if (!shouldContinue)
@@ -310,12 +301,16 @@ public class OAI2Fetcher implements EntryFetcher, Runnable {
                 /* update the dialogs progress bar */
                 dialog.setProgress(i + 1, keys.length);
             }
-            /* inform the inspection dialog, that we're done */
-            dialog.entryListComplete();
-            frame.output("");
+            
+            return true;
         } catch (Exception e) {
-            frame.output(Globals.lang("Error while fetching from OIA2")+ ": " + e.getMessage());
+            status.setStatus(Globals.lang("Error while fetching from OIA2")+ ": " + e.getMessage());
             e.printStackTrace();
         }
+        return false;
+    }
+
+    public void stopFetching() {
+        shouldContinue = false;
     }
 }
