@@ -27,29 +27,41 @@ http://www.gnu.org/copyleft/gpl.ja.html
 package net.sf.jabref.imports;
 
 import java.io.File;
+import net.sf.jabref.BibtexEntryType;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
-
 import net.sf.jabref.BibtexDatabase;
+import net.sf.jabref.BibtexEntry;
 
 public class ParserResult {
 
     public static ParserResult INVALID_FORMAT = new ParserResult(null, null, null);
     private BibtexDatabase base;
-    private HashMap metaData, entryTypes;
+    private HashMap<String, String> metaData;
+    private HashMap<String, BibtexEntryType> entryTypes;
+
+
     private File file = null;
     private ArrayList<String> warnings = new ArrayList<String>();
+    private String errorMessage = null;
     private String encoding = null; // Which encoding was used?
 
+    private boolean postponedAutosaveFound = false;
+    private boolean invalid = false;
 
     private String jabrefVersion = null; // Which JabRef version wrote the file, if any?
     private int jabrefMajorVersion = 0, jabrefMinorVersion = 0; // Numeric version representation
     private boolean toOpenTab = false;
 
-    public ParserResult(BibtexDatabase base, HashMap metaData, HashMap entryTypes) {
-	this.base = base;
-	this.metaData = metaData;
-	this.entryTypes = entryTypes;
+    public ParserResult(Collection<BibtexEntry> entries){
+    	this(ImportFormatReader.createDatabase(entries), null, new HashMap<String, BibtexEntryType>());
+    }
+    
+    public ParserResult(BibtexDatabase base, HashMap<String, String> metaData, HashMap<String, BibtexEntryType> entryTypes) {
+		this.base = base;
+		this.metaData = metaData;
+		this.entryTypes = entryTypes;
     }
 
     /**
@@ -102,12 +114,12 @@ public class ParserResult {
     	return base;
     }
 
-    public HashMap getMetaData() {
+    public HashMap<String, String> getMetaData() {
 	return metaData;
     }
 
-    public HashMap getEntryTypes() {
-	return entryTypes;
+    public HashMap<String, BibtexEntryType> getEntryTypes() {
+    	return entryTypes;
     }
 
     public File getFile() {
@@ -152,8 +164,31 @@ public class ParserResult {
     public String[] warnings() {
       String[] s = new String[warnings.size()];
       for (int i=0; i<warnings.size(); i++)
-        s[i] = (String)warnings.get(i);
+        s[i] = warnings.get(i);
       return s;
     }
 
+    public boolean isPostponedAutosaveFound() {
+        return postponedAutosaveFound;
+    }
+
+    public void setPostponedAutosaveFound(boolean postponedAutosaveFound) {
+        this.postponedAutosaveFound = postponedAutosaveFound;
+    }
+
+    public boolean isInvalid() {
+        return invalid;
+    }
+
+    public void setInvalid(boolean invalid) {
+        this.invalid = invalid;
+    }
+
+    public String getErrorMessage() {
+        return errorMessage;
+    }
+
+    public void setErrorMessage(String errorMessage) {
+        this.errorMessage = errorMessage;
+    }
 }
