@@ -1,29 +1,31 @@
 package net.sf.jabref.journals;
 
-import net.sf.jabref.*;
-import net.sf.jabref.net.URLDownload;
-
-import javax.swing.*;
-import javax.swing.table.AbstractTableModel;
-
-import com.jgoodies.forms.layout.FormLayout;
-import com.jgoodies.forms.layout.CellConstraints;
-import com.jgoodies.forms.builder.DefaultFormBuilder;
-import com.jgoodies.forms.builder.ButtonBarBuilder;
-import com.jgoodies.forms.builder.ButtonStackBuilder;
-
+import java.awt.BorderLayout;
+import java.awt.Dimension;
 import java.awt.event.*;
-import java.awt.*;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FileWriter;
 import java.io.IOException;
-import java.util.Map;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.Collections;
-import java.net.URL;
 import java.net.MalformedURLException;
+import java.net.URL;
+import java.util.*;
+
+import javax.swing.*;
+import javax.swing.table.AbstractTableModel;
+
+import net.sf.jabref.GUIGlobals;
+import net.sf.jabref.Globals;
+import net.sf.jabref.HelpAction;
+import net.sf.jabref.JabRefFrame;
+import net.sf.jabref.gui.FileDialogs;
+import net.sf.jabref.net.URLDownload;
+
+import com.jgoodies.forms.builder.ButtonBarBuilder;
+import com.jgoodies.forms.builder.ButtonStackBuilder;
+import com.jgoodies.forms.builder.DefaultFormBuilder;
+import com.jgoodies.forms.layout.CellConstraints;
+import com.jgoodies.forms.layout.FormLayout;
 
 /**
  * Created by IntelliJ IDEA.
@@ -45,7 +47,7 @@ public class ManageJournalsPanel extends JPanel{
     JTextField nameTf = new JTextField(),
         newNameTf = new JTextField(),
         abbrTf = new JTextField();
-    java.util.List externals = new ArrayList(); // To hold references to external journal lists.
+    List<ExternalFileEntry> externals = new ArrayList<ExternalFileEntry>(); // To hold references to external journal lists.
     JDialog dialog;
     JRadioButton newFile = new JRadioButton(Globals.lang("New file")),
         oldFile = new JRadioButton(Globals.lang("Existing file"));
@@ -143,7 +145,7 @@ public class ManageJournalsPanel extends JPanel{
                 File old = null;
                 if (!newNameTf.getText().equals(""))
                     old = new File(newNameTf.getText());
-                String name = Globals.getNewFile(frame, old, null, JFileChooser.SAVE_DIALOG, false);
+                String name = FileDialogs.getNewFile(frame, old, null, JFileChooser.SAVE_DIALOG, false);
                 if (name != null) {
                     if ((old != null) && (tableModel.getRowCount() > 0)) {
                     }
@@ -157,7 +159,7 @@ public class ManageJournalsPanel extends JPanel{
                 File old = null;
                 if (!personalFile.getText().equals(""))
                     old = new File(personalFile.getText());
-                String name = Globals.getNewFile(frame, old, null, JFileChooser.OPEN_DIALOG, false);
+                String name = FileDialogs.getNewFile(frame, old, null, JFileChooser.OPEN_DIALOG, false);
                 if (name != null) {
                     if ((old != null) && (tableModel.getRowCount() > 0)) {
                     }
@@ -229,8 +231,8 @@ public class ManageJournalsPanel extends JPanel{
     private void buildExternalsPanel() {
 
         DefaultFormBuilder builder = new DefaultFormBuilder(new FormLayout("fill:pref:grow",""));
-        for (Iterator i=externals.iterator(); i.hasNext();) {
-            ExternalFileEntry efe = (ExternalFileEntry)i.next();
+        for (Iterator<ExternalFileEntry> i=externals.iterator(); i.hasNext();) {
+            ExternalFileEntry efe = i.next();
             builder.append(efe.getPanel());
             builder.nextLine();
         }
@@ -327,8 +329,8 @@ public class ManageJournalsPanel extends JPanel{
             FileWriter fw = null;
             try {
                 fw = new FileWriter(f, false);
-                for (Iterator i=tableModel.getJournals().iterator(); i.hasNext();) {
-                    JournalEntry entry = (JournalEntry)i.next();
+                for (Iterator<JournalEntry> i=tableModel.getJournals().iterator(); i.hasNext();) {
+                    JournalEntry entry = i.next();
                     fw.write(entry.name);
                     fw.write(" = ");
                     fw.write(entry.abbreviation);
@@ -354,9 +356,9 @@ public class ManageJournalsPanel extends JPanel{
         }
 
         // Store the list of external files set up:
-        ArrayList extFiles = new ArrayList();
-        for (Iterator i=externals.iterator(); i.hasNext();) {
-            ExternalFileEntry efe = (ExternalFileEntry)i.next();
+        ArrayList<String> extFiles = new ArrayList<String>();
+        for (Iterator<ExternalFileEntry> i=externals.iterator(); i.hasNext();) {
+            ExternalFileEntry efe = i.next();
             if (!efe.getValue().equals("")) {
                 extFiles.add(efe.getValue());
             }
@@ -364,7 +366,7 @@ public class ManageJournalsPanel extends JPanel{
         if (extFiles.size() == 0)
             Globals.prefs.put("externalJournalLists", "");
         else {
-            String[] list = (String[])extFiles.toArray(new String[extFiles.size()]);
+            String[] list = extFiles.toArray(new String[extFiles.size()]);
             Globals.prefs.putStringArray("externalJournalLists", list);
         }
 
@@ -390,7 +392,7 @@ public class ManageJournalsPanel extends JPanel{
             File toFile;
             try {
                 URL url = new URL(chosen);
-                String toName = Globals.getNewFile(frame, new File(System.getProperty("user.home")),
+                String toName = FileDialogs.getNewFile(frame, new File(System.getProperty("user.home")),
                         null, JFileChooser.SAVE_DIALOG, false);
                 if (toName == null)
                     return;
@@ -419,10 +421,10 @@ public class ManageJournalsPanel extends JPanel{
         public void actionPerformed(ActionEvent e) {
             String chosen = null;
             if (dir)
-                chosen = Globals.getNewDir(frame, new File(comp.getText()), Globals.NONE,
+                chosen = FileDialogs.getNewDir(frame, new File(comp.getText()), Globals.NONE,
                         JFileChooser.OPEN_DIALOG, false);
             else
-                chosen = Globals.getNewFile(frame, new File(comp.getText()), Globals.NONE,
+                chosen = FileDialogs.getNewFile(frame, new File(comp.getText()), Globals.NONE,
                         JFileChooser.OPEN_DIALOG, false);
             if (chosen != null) {
                 File newFile = new File(chosen);
@@ -434,24 +436,22 @@ public class ManageJournalsPanel extends JPanel{
     class AbbreviationsTableModel extends AbstractTableModel implements ActionListener {
 
         String[] names = new String[] {Globals.lang("Journal name"), Globals.lang("Abbreviation")};
-        ArrayList journals = null;
+        ArrayList<JournalEntry> journals = null;
 
         public AbbreviationsTableModel() {
 
 
         }
 
-        public void setJournals(Map journals) {
-            this.journals = new ArrayList();
-            for (Iterator i=journals.keySet().iterator(); i.hasNext();) {
-                String journal = (String)i.next(),
-                        abbr = (String)journals.get(journal);
-                this.journals.add(new JournalEntry(journal, abbr));
+        public void setJournals(Map<String, String> journals) {
+            this.journals = new ArrayList<JournalEntry>();
+            for (Map.Entry<String, String> entry : journals.entrySet()){
+                this.journals.add(new JournalEntry(entry.getKey(), entry.getValue()));
             }
             fireTableDataChanged();
         }
 
-        public ArrayList getJournals() {
+        public ArrayList<JournalEntry> getJournals() {
             return journals;
         }
 
@@ -465,13 +465,13 @@ public class ManageJournalsPanel extends JPanel{
 
         public Object getValueAt(int row, int col) {
             if (col == 0)
-                return ((JournalEntry)journals.get(row)).name;
+                return journals.get(row).name;
             else
-                return ((JournalEntry)journals.get(row)).abbreviation;
+                return journals.get(row).abbreviation;
         }
 
         public void setValueAt(Object object, int row, int col) {
-            JournalEntry entry = (JournalEntry)journals.get(row);
+            JournalEntry entry = journals.get(row);
             if (col == 0)
                 entry.name = (String)object;
             else
@@ -594,15 +594,14 @@ public class ManageJournalsPanel extends JPanel{
         public String getValue() { return tf.getText(); }
     }
 
-    class JournalEntry implements Comparable {
+    class JournalEntry implements Comparable<JournalEntry> {
         String name, abbreviation;
         public JournalEntry(String name, String abbreviation) {
             this.name = name;
             this.abbreviation = abbreviation;
         }
-        public int compareTo(Object other) {
-            JournalEntry entry = (JournalEntry)other;
-            return this.name.compareTo(entry.name);
+        public int compareTo(JournalEntry other) {
+            return this.name.compareTo(other.name);
         }
     }
 }
