@@ -27,7 +27,6 @@ http://www.gnu.org/copyleft/gpl.ja.html
 package net.sf.jabref.label;
 
 import net.sf.jabref.BibtexEntry;
-import net.sf.jabref.Globals;
 import net.sf.jabref.BibtexFields;
 
 public class DefaultLabelRule implements LabelRule {
@@ -35,7 +34,7 @@ public class DefaultLabelRule implements LabelRule {
 
     // the default rule is that oit simply returns what it was given
     public String applyRule(BibtexEntry oldEntry){
-        return (String)oldEntry.getField(BibtexFields.KEY_FIELD);
+        return oldEntry.getField(BibtexFields.KEY_FIELD);
     }
 
 
