@@ -26,8 +26,6 @@ http://www.gnu.org/copyleft/gpl.ja.html
 */
 package net.sf.jabref.label;
 
-import net.sf.jabref.*;
-import java.util.StringTokenizer ; 
 
 public class IncollectionLabelRule extends ArticleLabelRule {
 
