@@ -1,26 +1,32 @@
 package net.sf.jabref.mods;
-import net.sf.jabref.*;
-import net.sf.jabref.export.layout.format.*;
-import net.sf.jabref.export.layout.*;
-
-import javax.xml.parsers.*;
-import javax.xml.transform.*;
-import javax.xml.transform.dom.*;
-import javax.xml.transform.stream.*;
-import org.w3c.dom.*;
+import java.io.StringWriter;
 import java.util.*;
-import java.io.*;
+
+import javax.xml.parsers.DocumentBuilder;
+import javax.xml.parsers.DocumentBuilderFactory;
+import javax.xml.transform.OutputKeys;
+import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerFactory;
+import javax.xml.transform.dom.DOMSource;
+import javax.xml.transform.stream.StreamResult;
+
+import net.sf.jabref.BibtexEntry;
+import net.sf.jabref.BibtexEntryType;
+import net.sf.jabref.export.layout.LayoutFormatter;
+import net.sf.jabref.export.layout.format.XMLChars;
+
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+import org.w3c.dom.Node;
 /**
  * @author Michael Wrighton
  *
- * TODO To change the template for this generated type comment go to
- * Window - Preferences - Java - Code Style - Code Templates
  */
 public class MODSEntry {
 	protected String entryType = "mods"; // could also be relatedItem
 	protected String id;
-	protected List authors = null;
-	protected List editors = null;
+	protected List<PersonName> authors = null;
+
 	// should really be handled with an enum
 	protected String issuance = "monographic";
 	protected PageNumbers pages = null;
@@ -36,18 +42,18 @@ public class MODSEntry {
 	protected String number;
 	protected String volume;
 	protected String genre = null;
-	protected Set handledExtensions;
+	protected Set<String> handledExtensions;
 	
 	protected MODSEntry host;
-	Map extensionFields;
+	Map<String, String> extensionFields;
 	
 	public static String BIBTEX = "bibtex_";
 	
 	private final boolean CHARFORMAT = false;
 	
 	public MODSEntry() {
-		extensionFields = new HashMap();
-		handledExtensions = new HashSet();
+		extensionFields = new HashMap<String, String>();
+		handledExtensions = new HashSet<String>();
 	
 	}
 	
@@ -94,14 +100,14 @@ public class MODSEntry {
 		{
 			host = new MODSEntry();
 			host.entryType = "relatedItem";
-			host.title = (String) bibtex.getField("booktitle");
-			host.publisher = (String) bibtex.getField("publisher");
-			host.number = (String) bibtex.getField("number");
+			host.title = bibtex.getField("booktitle");
+			host.publisher = bibtex.getField("publisher");
+			host.number = bibtex.getField("number");
 			if (bibtex.getField("pages") != null)
-				host.volume = (String) bibtex.getField("volume");
+				host.volume = bibtex.getField("volume");
 			host.issuance = "continuing";
 			if (bibtex.getField("pages") != null)
-				host.pages = new PageNumbers((String) bibtex.getField("pages"));
+				host.pages = new PageNumbers(bibtex.getField("pages"));
 		}
 		
 		populateExtensionFields(bibtex);
@@ -109,17 +115,16 @@ public class MODSEntry {
 	}
 	
 	protected void populateExtensionFields(BibtexEntry e) {
-		Object fields [] = e.getAllFields();
-		for(int i = 0; i < fields.length; i++) {
-			String field = (String) fields[i];
-			String value = (String) e.getField(field);
+		
+		for (String field : e.getAllFields()){
+			String value = e.getField(field);
 			field = BIBTEX + field;
 			extensionFields.put(field, value);
 		}
 	}
 	
-	protected List getAuthors(String authors) {
-		List result = new LinkedList();
+	protected List<PersonName> getAuthors(String authors) {
+		List<PersonName> result = new LinkedList<PersonName>();
 		LayoutFormatter chars = new XMLChars();
 		
 		if (authors.indexOf(" and ") == -1) {
@@ -154,21 +159,19 @@ public class MODSEntry {
 	// must be from http://www.loc.gov/marc/sourcecode/genre/genrelist.html
 	protected String getMODSgenre(BibtexEntry bibtex) {
 		String bibtexType = bibtex.getType().getName();
-		String result;
-		if (bibtexType.equals("Mastersthesis"))
-			result = "theses";
-		else
-			result = "conference publication";
-		// etc...
+		/**
+		 * <pre> String result; if (bibtexType.equals("Mastersthesis")) result =
+		 * "theses"; else result = "conference publication"; // etc... </pre>
+		 */
 		return bibtexType;		
 	}
 	
-	public Document getDOMrepresentation() {
-		Document result = null;
+	public Node getDOMrepresentation() {
+		Node result = null;
 		try {
 			DocumentBuilder d = DocumentBuilderFactory.newInstance().newDocumentBuilder();
 			
-		//	result = getDOMrepresentation(d);
+			result = getDOMrepresentation(d.newDocument());
 		}
 		catch (Exception e) 
 		{
@@ -179,7 +182,6 @@ public class MODSEntry {
 		
 	
 	public Element getDOMrepresentation(Document d) {
-		Node result = null;
 	   	try {
 	   		Element mods = d.createElement(entryType);
 	   		mods.setAttribute("version", "3.0");
@@ -193,8 +195,8 @@ public class MODSEntry {
 		   		mods.appendChild(titleInfo);
 	   		}
 	   		if (authors != null) {
-	   			for(Iterator iter = authors.iterator(); iter.hasNext();) {
-	   				PersonName name = (PersonName) iter.next();
+	   			for(Iterator<PersonName> iter = authors.iterator(); iter.hasNext();) {
+	   				PersonName name = iter.next();
 	   				Element modsName = d.createElement("name");
 	   				modsName.setAttribute("type", "personal");
 	   				if (name.getSurname() != null) {
@@ -263,11 +265,10 @@ public class MODSEntry {
 	   		}
 	   		
 	   		/* now generate extension fields for unhandled data */
-	   		for(Iterator iter = extensionFields.entrySet().iterator(); iter.hasNext(); ) {
+	   		for(Map.Entry<String, String> theEntry : extensionFields.entrySet()){
 	   			Element extension = d.createElement("extension");
-	   			Map.Entry theEntry = (Map.Entry) iter.next();
-	   			String field = (String) theEntry.getKey();
-	   			String value = (String) theEntry.getValue();
+	   			String field = theEntry.getKey();
+	   			String value = theEntry.getValue();
 	   			if (handledExtensions.contains(field))
 	   				continue;
 	   			Element theData = d.createElement(field);
@@ -319,6 +320,7 @@ public class MODSEntry {
 
 	/*
 	 * render as XML
+	 * 
 	 */
 	public String toString() {
 		StringWriter sresult = new StringWriter();
