@@ -5,15 +5,10 @@
 package net.sf.jabref.net;
 
 import java.awt.Component;
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
-import java.io.File;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
+import java.io.*;
 import java.net.URL;
 import java.net.URLConnection;
+import java.net.CookieHandler;
 
 import javax.swing.ProgressMonitorInputStream;
 
@@ -24,18 +19,48 @@ import javax.swing.ProgressMonitorInputStream;
 public class URLDownload {  
     
     private URL source;
+    private URLConnection con = null;
     private File dest;
     private Component parent;
+    private String mimeType = null;
+
+    private CookieHandler cm;
 
     public URLDownload(Component _parent, URL _source, File _dest) {
         source = _source;
         dest = _dest;
         parent = _parent;
+
+        try {
+            // This should set up JabRef to receive cookies properly
+            if ((cm = CookieHandler.getDefault()) == null) {
+                cm = new CookieHandlerImpl();
+                CookieHandler.setDefault(cm);
+            }
+        } catch (SecurityException e) {
+            // Setting or getting the system default cookie handler is forbidden
+            // In this case cookie handling is not possible.
+        }
     }
-    
-    public void download() throws IOException {
-        URLConnection con = source.openConnection();
+
+    public String getMimeType() {
+        return mimeType;
+    }
+
+    public void openConnectionOnly() throws IOException {
+        con = source.openConnection();
         con.setRequestProperty("User-Agent", "Jabref");
+        mimeType = con.getContentType();
+    }
+
+    public void download() throws IOException {
+
+        if (con == null) {
+            con = source.openConnection();
+            con.setRequestProperty("User-Agent", "Jabref");
+            mimeType = con.getContentType();
+        }
+
     	InputStream input = new BufferedInputStream(con.getInputStream());
         OutputStream output =  new BufferedOutputStream(new FileOutputStream(dest));
      
@@ -64,7 +89,6 @@ public class URLDownload {
       {
         InputStream _in = new ProgressMonitorInputStream(parent, "Downloading " + source.toString(), in);
         byte[] buffer = new byte[512];
-        int reps=0;
         while(true)
         {
             int bytesRead = _in.read(buffer);
