@@ -2,7 +2,7 @@ package net.sf.jabref.util;
 
 /* Mp3dings - manage mp3 meta-information
  * Copyright (C) 2003 Moritz Ringler
- * $Id: CaseChangeMenu.java 864 2005-03-20 20:47:54Z mortenalver $
+ * $Id: CaseChangeMenu.java 2268 2007-08-19 23:37:05Z coezbek $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,10 +37,10 @@ public class CaseChangeMenu extends JMenu implements ActionListener{
         /* case */
         super(Globals.lang("Change case"));
         parent = opener;
-        int m = cc.getNumModes();
+        int m = CaseChanger.getNumModes();
         changeCaseItems = new JMenuItem[m];
         for (int i=0;i<m;i++){
-            changeCaseItems[i]=new JMenuItem(Globals.lang(cc.getModeName(i)));
+            changeCaseItems[i]=new JMenuItem(Globals.lang(CaseChanger.getModeName(i)));
             changeCaseItems[i].addActionListener(this);
             this.add(changeCaseItems[i]);
         }
@@ -48,7 +48,7 @@ public class CaseChangeMenu extends JMenu implements ActionListener{
 
     public void actionPerformed(ActionEvent e) {
         Object source = (e.getSource());
-        for(int i=0, m=cc.getNumModes(); i<m; i++){
+        for(int i=0, m=CaseChanger.getNumModes(); i<m; i++){
             if(source == changeCaseItems[i]){
                 caseChange(i);
                 break;
@@ -57,6 +57,6 @@ public class CaseChangeMenu extends JMenu implements ActionListener{
     }
 
     private void caseChange(int mode){
-        parent.setText(cc.changeCase(parent.getText(), mode));
+        parent.setText(CaseChanger.changeCase(parent.getText(), mode));
     }
 }
\ No newline at end of file
