@@ -50,19 +50,28 @@
 
 package net.sf.jabref.wizard.text.gui ;
 
-import java.io.* ;
-import java.net.* ;
+import java.awt.*;
+import java.awt.event.*;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.IOException;
+import java.io.StringWriter;
+import java.net.URL;
 
-import java.awt.* ;
-import java.awt.event.* ;
-import javax.swing.* ;
-import javax.swing.border.* ;
-import javax.swing.event.* ;
-import javax.swing.text.* ;
+import javax.swing.*;
+import javax.swing.border.EtchedBorder;
+import javax.swing.border.TitledBorder;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
+import javax.swing.event.ListSelectionEvent;
+import javax.swing.event.ListSelectionListener;
+import javax.swing.text.*;
+
+import net.sf.jabref.*;
+import net.sf.jabref.gui.FileDialogs;
+import net.sf.jabref.wizard.integrity.gui.IntegrityMessagePanel;
+import net.sf.jabref.wizard.text.TagToMarkedTextStore;
 
-import net.sf.jabref.* ;
-import net.sf.jabref.wizard.integrity.gui.* ;
-import net.sf.jabref.wizard.text.* ;
 import com.jgoodies.forms.builder.ButtonBarBuilder;
 
 public class TextInputDialog
@@ -161,20 +170,8 @@ public class TextInputDialog
     tabbed.add( warnPanel, Globals.lang( "Messages_and_Hints" ) ) ;
 
     // Panel Layout
-    GridBagLayout gbl = new GridBagLayout() ;
-    GridBagConstraints con = new GridBagConstraints() ;
-    con.weightx = 1 ;
-    con.weighty = 1 ;
-    con.insets = new Insets( 5, 10, 0, 10 ) ;
-    con.fill = GridBagConstraints.HORIZONTAL ;
-
     panel1.setLayout( new BorderLayout() ) ;
-
-    con.gridwidth = GridBagConstraints.REMAINDER ;
-    con.fill = GridBagConstraints.BOTH;
-    //gbl.setConstraints( tabbed, con ) ;
     panel1.add( tabbed, BorderLayout.CENTER ) ;
-    //gbl.setConstraints( buttons, con ) ;
     panel1.add( buttons, BorderLayout.SOUTH ) ;
 
     // Key bindings:
@@ -478,7 +475,7 @@ public class TextInputDialog
           marked.appendPosition( type, selStart, selEnd ) ;
 
           // get old text from bibtex tag
-          String old = ( String ) entry.getField( type ) ;
+          String old = entry.getField( type ) ;
 
           // merge old and selected text
           if ( old != null )
@@ -631,7 +628,7 @@ public class TextInputDialog
       try
       {
         String chosen = null ;
-        chosen = Globals.getNewFile( _frame, null, null,
+        chosen = FileDialogs.getNewFile( _frame, null, null,
                                      ".txt",
                                      JFileChooser.OPEN_DIALOG, false ) ;
         if ( chosen != null )
